@@ -4,12 +4,12 @@
 
 import {
   hasInScopeLines,
-  getLocationSource,
   getSourceTextContent,
   getVisibleSelectedFrame,
-} from "../../selectors";
+} from "../../selectors/index";
 
 import { getSourceLineCount } from "../../utils/source";
+import { features } from "../../utils/prefs";
 
 import { isFulfilled } from "../../utils/async-value";
 
@@ -28,17 +28,10 @@ function getOutOfScopeLines(outOfScopeLocations) {
   return uniqueLines;
 }
 
-async function getInScopeLines(
-  cx,
-  location,
-  { dispatch, getState, parserWorker }
-) {
-  const source = getLocationSource(getState(), location);
-  const sourceTextContent = getSourceTextContent(getState(), location);
-
+async function getInScopeLines(location, sourceTextContent, { parserWorker }) {
   let locations = null;
-  if (location.line && source && !source.isWasm) {
-    locations = await parserWorker.findOutOfScopeLocations(source.id, location);
+  if (location.line && parserWorker.isLocationSupported(location)) {
+    locations = await parserWorker.findOutOfScopeLocations(location);
   }
 
   const linesOutOfScope = getOutOfScopeLines(locations);
@@ -67,8 +60,13 @@ async function getInScopeLines(
   // out of scope lines.
   return sourceLines.filter(i => i != undefined);
 }
-
-export function setInScopeLines(cx) {
+/**
+ * Get and store the in scope lines in the reducer
+ * @param {Object} editor - The editor provides an API to retrieve the in scope location
+ *                          details based on lezer in CM6.
+ * @returns
+ */
+export function setInScopeLines(editor) {
   return async thunkArgs => {
     const { getState, dispatch } = thunkArgs;
     const visibleFrame = getVisibleSelectedFrame(getState());
@@ -80,15 +78,25 @@ export function setInScopeLines(cx) {
     const { location } = visibleFrame;
     const sourceTextContent = getSourceTextContent(getState(), location);
 
-    if (hasInScopeLines(getState(), location) || !sourceTextContent) {
+    // Ignore if in scope lines have already be computed, or if the selected location
+    // doesn't have its content already fully fetched.
+    // The ParserWorker will only have the source text content once the source text content is fulfilled.
+    if (
+      hasInScopeLines(getState(), location) ||
+      !sourceTextContent ||
+      !isFulfilled(sourceTextContent) ||
+      !editor
+    ) {
       return;
     }
 
-    const lines = await getInScopeLines(cx, location, thunkArgs);
+    const lines =
+      features.codemirrorNext && editor
+        ? await editor.getInScopeLines(location)
+        : await getInScopeLines(location, sourceTextContent, thunkArgs);
 
     dispatch({
       type: "IN_SCOPE_LINES",
-      cx,
       location,
       lines,
     });
