@@ -2,12 +2,14 @@
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at <http://mozilla.org/MPL/2.0/>. */
 
-import { clearDocuments } from "../utils/editor";
 import sourceQueue from "../utils/source-queue";
 
 import { clearWasmStates } from "../utils/wasm";
-import { getMainThread, getThreadContext } from "../selectors";
-import { evaluateExpressions } from "../actions/expressions";
+import { getMainThread } from "../selectors/index";
+import { evaluateExpressionsForCurrentContext } from "../actions/expressions";
+
+import { features } from "../utils/prefs";
+import { getEditor } from "../utils/editor/index";
 
 /**
  * Redux actions for the navigation state
@@ -19,17 +21,14 @@ import { evaluateExpressions } from "../actions/expressions";
  * @static
  */
 export function willNavigate(event) {
-  return async function ({
-    dispatch,
-    getState,
-    client,
-    sourceMapLoader,
-    parserWorker,
-  }) {
+  return async function ({ dispatch, getState, sourceMapLoader }) {
     sourceQueue.clear();
     sourceMapLoader.clearSourceMaps();
+    if (features.codemirrorNext) {
+      const editor = getEditor(features.codemirrorNext);
+      editor.clearSources();
+    }
     clearWasmStates();
-    clearDocuments();
     const thread = getMainThread(getState());
 
     dispatch({
@@ -44,11 +43,10 @@ export function willNavigate(event) {
  * @static
  */
 export function navigated() {
-  return async function ({ getState, dispatch, panel }) {
+  return async function ({ dispatch, panel }) {
     try {
       // Update the watched expressions once the page is fully loaded
-      const threadcx = getThreadContext(getState());
-      await dispatch(evaluateExpressions(threadcx));
+      await dispatch(evaluateExpressionsForCurrentContext());
     } catch (e) {
       // This may throw if we resume during the page load.
       // browser_dbg-debugger-buttons.js highlights this, especially on MacOS or when ran many times
