@@ -11,16 +11,19 @@ import {
   getBreakpointsForSource,
   getSourceActorsForSource,
   getFirstSourceActorForGeneratedSource,
-} from "../../selectors";
-import { addBreakpoint } from "../breakpoints";
+} from "../../selectors/index";
+import { addBreakpoint } from "../breakpoints/index";
 
-import { prettyPrintSource } from "./prettyPrint";
+import { prettyPrintSourceTextContent } from "./prettyPrint";
 import { isFulfilled, fulfilled } from "../../utils/async-value";
 
 import { isPretty } from "../../utils/source";
 import { createLocation } from "../../utils/location";
 import { memoizeableAction } from "../../utils/memoizableAction";
 
+import { features } from "../../utils/prefs";
+import { getEditor } from "../../utils/editor/index";
+
 async function loadGeneratedSource(sourceActor, { client }) {
   // If no source actor can be found then the text for the
   // source cannot be loaded.
@@ -43,7 +46,7 @@ async function loadGeneratedSource(sourceActor, { client }) {
 
 async function loadOriginalSource(
   source,
-  { getState, client, sourceMapLoader, prettyPrintWorker }
+  { getState, sourceMapLoader, prettyPrintWorker }
 ) {
   if (isPretty(source)) {
     const generatedSource = getGeneratedSource(getState(), source);
@@ -54,11 +57,11 @@ async function loadOriginalSource(
     const content = getSettledSourceTextContent(
       getState(),
       createLocation({
-        sourceId: generatedSource.id,
+        source: generatedSource,
       })
     );
 
-    return prettyPrintSource(
+    return prettyPrintSourceTextContent(
       sourceMapLoader,
       prettyPrintWorker,
       generatedSource,
@@ -78,76 +81,87 @@ async function loadOriginalSource(
   return result;
 }
 
-async function loadGeneratedSourceTextPromise(cx, sourceActor, thunkArgs) {
+async function loadGeneratedSourceTextPromise(sourceActor, thunkArgs) {
   const { dispatch, getState } = thunkArgs;
   const epoch = getSourcesEpoch(getState());
 
   await dispatch({
     type: "LOAD_GENERATED_SOURCE_TEXT",
-    sourceActorId: sourceActor.actor,
+    sourceActor,
     epoch,
     [PROMISE]: loadGeneratedSource(sourceActor, thunkArgs),
   });
 
   await onSourceTextContentAvailable(
-    cx,
     sourceActor.sourceObject,
     sourceActor,
     thunkArgs
   );
 }
 
-async function loadOriginalSourceTextPromise(cx, source, thunkArgs) {
+async function loadOriginalSourceTextPromise(source, thunkArgs) {
   const { dispatch, getState } = thunkArgs;
   const epoch = getSourcesEpoch(getState());
   await dispatch({
     type: "LOAD_ORIGINAL_SOURCE_TEXT",
-    sourceId: source.id,
+    source,
     epoch,
     [PROMISE]: loadOriginalSource(source, thunkArgs),
   });
 
-  await onSourceTextContentAvailable(cx, source, null, thunkArgs);
+  await onSourceTextContentAvailable(source, null, thunkArgs);
 }
 
 /**
  * Function called everytime a new original or generated source gets its text content
  * fetched from the server and registered in the reducer.
  *
- * @param {Object} cx
  * @param {Object} source
  * @param {Object} sourceActor (optional)
  *        If this is a generated source, we expect a precise source actor.
  * @param {Object} thunkArgs
  */
 async function onSourceTextContentAvailable(
-  cx,
   source,
   sourceActor,
   { dispatch, getState, parserWorker }
 ) {
-  const content = getSettledSourceTextContent(
-    getState(),
-    createLocation({
-      sourceId: source.id,
-      sourceActorId: sourceActor?.id,
-    })
-  );
+  const location = createLocation({
+    source,
+    sourceActor,
+  });
+  const content = getSettledSourceTextContent(getState(), location);
+  if (!content) {
+    return;
+  }
 
-  if (!source.isWasm && content) {
-    parserWorker.setSource(
-      source.id,
-      isFulfilled(content)
-        ? content.value
-        : { type: "text", value: "", contentType: undefined }
-    );
+  const contentValue = isFulfilled(content)
+    ? content.value
+    : { type: "text", value: "", contentType: undefined };
 
-    // Update the text in any breakpoints for this source by re-adding them.
-    const breakpoints = getBreakpointsForSource(getState(), source.id);
-    for (const { location, options, disabled } of breakpoints) {
-      await dispatch(addBreakpoint(cx, location, options, disabled));
+  // Lezer parser uses the sources to map the original frame names
+  if (features.codemirrorNext) {
+    const editor = getEditor(features.codemirrorNext);
+    if (!editor.isWasm) {
+      editor.addSource(source.id, contentValue.value);
     }
   }
+  // The babel parser needs the sources to parse scopes
+  if (parserWorker.isLocationSupported(location)) {
+    parserWorker.setSource(source.id, contentValue);
+  }
+
+  // Update the text in any breakpoints for this source by re-adding them.
+  const breakpoints = getBreakpointsForSource(getState(), source);
+  for (const breakpoint of breakpoints) {
+    await dispatch(
+      addBreakpoint(
+        breakpoint.location,
+        breakpoint.options,
+        breakpoint.disabled
+      )
+    );
+  }
 }
 
 /**
@@ -161,7 +175,7 @@ async function onSourceTextContentAvailable(
 export const loadGeneratedSourceText = memoizeableAction(
   "loadGeneratedSourceText",
   {
-    getValue: ({ sourceActor }, { getState }) => {
+    getValue: (sourceActor, { getState }) => {
       if (!sourceActor) {
         return null;
       }
@@ -169,8 +183,8 @@ export const loadGeneratedSourceText = memoizeableAction(
       const sourceTextContent = getSourceTextContent(
         getState(),
         createLocation({
-          sourceId: sourceActor.source,
-          sourceActorId: sourceActor.actor,
+          source: sourceActor.sourceObject,
+          sourceActor,
         })
       );
 
@@ -183,12 +197,12 @@ export const loadGeneratedSourceText = memoizeableAction(
       // propagate that error upward.
       return fulfilled(sourceTextContent);
     },
-    createKey: ({ sourceActor }, { getState }) => {
+    createKey: (sourceActor, { getState }) => {
       const epoch = getSourcesEpoch(getState());
       return `${epoch}:${sourceActor.actor}`;
     },
-    action: ({ cx, sourceActor }, thunkArgs) =>
-      loadGeneratedSourceTextPromise(cx, sourceActor, thunkArgs),
+    action: (sourceActor, thunkArgs) =>
+      loadGeneratedSourceTextPromise(sourceActor, thunkArgs),
   }
 );
 
@@ -200,7 +214,7 @@ export const loadGeneratedSourceText = memoizeableAction(
 export const loadOriginalSourceText = memoizeableAction(
   "loadOriginalSourceText",
   {
-    getValue: ({ source }, { getState }) => {
+    getValue: (source, { getState }) => {
       if (!source) {
         return null;
       }
@@ -208,7 +222,7 @@ export const loadOriginalSourceText = memoizeableAction(
       const sourceTextContent = getSourceTextContent(
         getState(),
         createLocation({
-          sourceId: source.id,
+          source,
         })
       );
       if (!sourceTextContent || sourceTextContent.state === "pending") {
@@ -220,30 +234,29 @@ export const loadOriginalSourceText = memoizeableAction(
       // propagate that error upward.
       return fulfilled(sourceTextContent);
     },
-    createKey: ({ source }, { getState }) => {
+    createKey: (source, { getState }) => {
       const epoch = getSourcesEpoch(getState());
       return `${epoch}:${source.id}`;
     },
-    action: ({ cx, source }, thunkArgs) =>
-      loadOriginalSourceTextPromise(cx, source, thunkArgs),
+    action: (source, thunkArgs) =>
+      loadOriginalSourceTextPromise(source, thunkArgs),
   }
 );
 
-export function loadSourceText(cx, source, sourceActor) {
+export function loadSourceText(source, sourceActor) {
   return async ({ dispatch, getState }) => {
     if (!source) {
       return null;
     }
     if (source.isOriginal) {
-      return dispatch(loadOriginalSourceText({ cx, source }));
+      return dispatch(loadOriginalSourceText(source));
     }
     if (!sourceActor) {
       sourceActor = getFirstSourceActorForGeneratedSource(
         getState(),
-        source.id,
-        source.thread
+        source.id
       );
     }
-    return dispatch(loadGeneratedSourceText({ cx, sourceActor }));
+    return dispatch(loadGeneratedSourceText(sourceActor));
   };
 }
