@@ -90,7 +90,6 @@ add_task(async function test_backgroundtask_debugger() {
     findElement,
     findElementWithSelector,
     getSelector,
-    getThreadContext,
     getVisibleSelectedFrameLine,
     isPaused,
     resume,
@@ -102,10 +101,13 @@ add_task(async function test_backgroundtask_debugger() {
     waitForPaused,
     waitForResumed,
     waitForSelectedSource,
+    waitForInlinePreviews,
     waitForState,
     waitUntil,
     createLocation,
-    getCM,
+    getEditorContent,
+    getCMEditor,
+    isCm6Enabled,
     log: (msg, data) =>
       console.log(`${msg} ${!data ? "" : JSON.stringify(data)}`),
     info: (msg, data) =>
