@@ -4,7 +4,10 @@
 
 // Test ensures zombie debug lines do not persist
 // https://github.com/firefox-devtools/debugger/issues/7755
-add_task(async function() {
+
+"use strict";
+
+add_task(async function () {
   // Load test files
   const dbg = await initDebugger("doc-sources.html");
   await waitForSources(dbg, "simple1.js", "simple2.js");
@@ -21,7 +24,9 @@ add_task(async function() {
   findElement(dbg, "frame", 2).focus();
 
   // Click the call stack to get to debugger-line-1
+  const dispatched = waitForDispatch(dbg.store, "ADD_INLINE_PREVIEW");
   await clickElement(dbg, "frame", 2);
+  await dispatched;
   await waitForRequestsToSettle(dbg);
   await waitForSelectedSource(dbg, "simple1.js");
 
@@ -33,7 +38,7 @@ add_task(async function() {
 
   info("Ensuring there's no zombie debug line");
   is(
-    findAllElements(dbg, "debugLine").length,
+    findAllElements(dbg, "pausedLine").length,
     0,
     "Debug line no longer exists!"
   );
