@@ -4,12 +4,10 @@
 
 "use strict";
 
-const promise = require("promise");
-const { KeyCodes } = require("devtools/client/shared/keycodes");
+const { KeyCodes } = require("resource://devtools/client/shared/keycodes.js");
 
-const EventEmitter = require("devtools/shared/event-emitter");
-const AutocompletePopup = require("devtools/client/shared/autocomplete-popup");
-const Services = require("Services");
+const EventEmitter = require("resource://devtools/shared/event-emitter.js");
+const AutocompletePopup = require("resource://devtools/client/shared/autocomplete-popup.js");
 
 // Maximum number of selector suggestions shown in the panel.
 const MAX_SUGGESTIONS = 15;
@@ -18,36 +16,44 @@ const MAX_SUGGESTIONS = 15;
  * Converts any input field into a document search box.
  *
  * @param {InspectorPanel} inspector
- *        The InspectorPanel whose `walker` attribute should be used for
- *        document traversal.
+ *        The InspectorPanel to access the inspector commands for
+ *        search and document traversal.
  * @param {DOMNode} input
  *        The input element to which the panel will be attached and from where
  *        search input will be taken.
  * @param {DOMNode} clearBtn
  *        The clear button in the input field that will clear the input value.
+ * @param {DOMNode} prevBtn
+ *        The prev button in the search label that will move
+ *        selection to previous match.
+ * @param {DOMNode} nextBtn
+ *        The next button in the search label that will move
+ *        selection to next match.
  *
  * Emits the following events:
  * - search-cleared: when the search box is emptied
  * - search-result: when a search is made and a result is selected
  */
-function InspectorSearch(inspector, input, clearBtn) {
+function InspectorSearch(inspector, input, clearBtn, prevBtn, nextBtn) {
   this.inspector = inspector;
   this.searchBox = input;
   this.searchClearButton = clearBtn;
+  this.searchPrevButton = prevBtn;
+  this.searchNextButton = nextBtn;
   this._lastSearched = null;
 
   this._onKeyDown = this._onKeyDown.bind(this);
   this._onInput = this._onInput.bind(this);
+  this.findPrev = this.findPrev.bind(this);
+  this.findNext = this.findNext.bind(this);
   this._onClearSearch = this._onClearSearch.bind(this);
 
   this.searchBox.addEventListener("keydown", this._onKeyDown, true);
   this.searchBox.addEventListener("input", this._onInput, true);
+  this.searchPrevButton.addEventListener("click", this.findPrev, true);
+  this.searchNextButton.addEventListener("click", this.findNext, true);
   this.searchClearButton.addEventListener("click", this._onClearSearch);
 
-  // For testing, we need to be able to wait for the most recent node request
-  // to finish.  Tests can watch this promise for that.
-  this._lastQuery = promise.resolve(null);
-
   this.autocompleter = new SelectorAutocompleter(inspector, input);
   EventEmitter.decorate(this);
 }
@@ -55,20 +61,20 @@ function InspectorSearch(inspector, input, clearBtn) {
 exports.InspectorSearch = InspectorSearch;
 
 InspectorSearch.prototype = {
-  get walker() {
-    return this.inspector.walker;
-  },
-
-  destroy: function() {
+  destroy() {
     this.searchBox.removeEventListener("keydown", this._onKeyDown, true);
     this.searchBox.removeEventListener("input", this._onInput, true);
+    this.searchPrevButton.removeEventListener("click", this.findPrev, true);
+    this.searchNextButton.removeEventListener("click", this.findNext, true);
     this.searchClearButton.removeEventListener("click", this._onClearSearch);
     this.searchBox = null;
+    this.searchPrevButton = null;
+    this.searchNextButton = null;
     this.searchClearButton = null;
     this.autocompleter.destroy();
   },
 
-  _onSearch: function(reverse = false) {
+  _onSearch(reverse = false) {
     this.doFullTextSearch(this.searchBox.value, reverse).catch(console.error);
   },
 
@@ -80,13 +86,18 @@ InspectorSearch.prototype = {
 
     if (query.length === 0) {
       searchContainer.classList.remove("devtools-searchbox-no-match");
-      if (!lastSearched || lastSearched.length > 0) {
+      if (!lastSearched || lastSearched.length) {
         this.emit("search-cleared");
       }
       return;
     }
 
-    const res = await this.walker.search(query, { reverse });
+    const res = await this.inspector.commands.inspectorCommand.findNextNode(
+      query,
+      {
+        reverse,
+      }
+    );
 
     // Value has changed since we started this request, we're done.
     if (query !== this.searchBox.value) {
@@ -98,7 +109,6 @@ InspectorSearch.prototype = {
         reason: "inspectorsearch",
       });
       searchContainer.classList.remove("devtools-searchbox-no-match");
-
       res.query = query;
       this.emit("search-result", res);
     } else {
@@ -107,7 +117,7 @@ InspectorSearch.prototype = {
     }
   },
 
-  _onInput: function() {
+  _onInput() {
     if (this.searchBox.value.length === 0) {
       this.searchClearButton.hidden = true;
       this._onSearch();
@@ -116,7 +126,7 @@ InspectorSearch.prototype = {
     }
   },
 
-  _onKeyDown: function(event) {
+  _onKeyDown(event) {
     if (event.keyCode === KeyCodes.DOM_VK_RETURN) {
       this._onSearch(event.shiftKey);
     }
@@ -129,9 +139,18 @@ InspectorSearch.prototype = {
     }
   },
 
-  _onClearSearch: function() {
+  findNext() {
+    this._onSearch();
+  },
+
+  findPrev() {
+    this._onSearch(true);
+  },
+
+  _onClearSearch() {
     this.searchBox.parentNode.classList.remove("devtools-searchbox-no-match");
     this.searchBox.value = "";
+    this.searchBox.focus();
     this.searchClearButton.hidden = true;
     this.emit("search-cleared");
   },
@@ -146,63 +165,54 @@ InspectorSearch.prototype = {
  *
  * @constructor
  * @param InspectorPanel inspector
- *        The InspectorPanel whose `walker` attribute should be used for
- *        document traversal.
+ *        The InspectorPanel to access the inspector commands for
+ *        search and document traversal.
  * @param nsiInputElement inputNode
  *        The input element to which the panel will be attached and from where
  *        search input will be taken.
  */
-function SelectorAutocompleter(inspector, inputNode) {
-  this.inspector = inspector;
-  this.searchBox = inputNode;
-  this.panelDoc = this.searchBox.ownerDocument;
+class SelectorAutocompleter extends EventEmitter {
+  constructor(inspector, inputNode) {
+    super();
 
-  this.showSuggestions = this.showSuggestions.bind(this);
-  this._onSearchKeypress = this._onSearchKeypress.bind(this);
-  this._onSearchPopupClick = this._onSearchPopupClick.bind(this);
-  this._onMarkupMutation = this._onMarkupMutation.bind(this);
+    this.inspector = inspector;
+    this.searchBox = inputNode;
+    this.panelDoc = this.searchBox.ownerDocument;
 
-  // Options for the AutocompletePopup.
-  const options = {
-    listId: "searchbox-panel-listbox",
-    autoSelect: true,
-    position: "top",
-    onClick: this._onSearchPopupClick,
-  };
+    this.showSuggestions = this.showSuggestions.bind(this);
 
-  // The popup will be attached to the toolbox document.
-  this.searchPopup = new AutocompletePopup(inspector._toolbox.doc, options);
+    // Options for the AutocompletePopup.
+    const options = {
+      listId: "searchbox-panel-listbox",
+      autoSelect: true,
+      position: "top",
+      onClick: this.#onSearchPopupClick,
+    };
 
-  this.searchBox.addEventListener("input", this.showSuggestions, true);
-  this.searchBox.addEventListener("keypress", this._onSearchKeypress, true);
-  this.inspector.on("markupmutation", this._onMarkupMutation);
+    // The popup will be attached to the toolbox document.
+    this.searchPopup = new AutocompletePopup(inspector._toolbox.doc, options);
 
-  // For testing, we need to be able to wait for the most recent node request
-  // to finish.  Tests can watch this promise for that.
-  this._lastQuery = promise.resolve(null);
-  EventEmitter.decorate(this);
-}
-
-exports.SelectorAutocompleter = SelectorAutocompleter;
-
-SelectorAutocompleter.prototype = {
-  get walker() {
-    return this.inspector.walker;
-  },
+    this.searchBox.addEventListener("input", this.showSuggestions, true);
+    this.searchBox.addEventListener("keypress", this.#onSearchKeypress, true);
+  }
 
   // The possible states of the query.
-  States: {
+  States = {
     CLASS: "class",
     ID: "id",
     TAG: "tag",
     ATTRIBUTE: "attribute",
-  },
+  };
 
   // The current state of the query.
-  _state: null,
+  #state = null;
 
   // The query corresponding to last state computation.
-  _lastStateCheckAt: null,
+  #lastStateCheckAt = null;
+
+  get walker() {
+    return this.inspector.walker;
+  }
 
   /**
    * Computes the state of the query. State refers to whether the query
@@ -214,20 +224,20 @@ SelectorAutocompleter.prototype = {
    *        '#f' requires an Id suggestion, so the state is States.ID
    *        'div > .foo' requires class suggestion, so state is States.CLASS
    */
-  /* eslint-disable complexity */
+  // eslint-disable-next-line complexity
   get state() {
     if (!this.searchBox || !this.searchBox.value) {
       return null;
     }
 
     const query = this.searchBox.value;
-    if (this._lastStateCheckAt == query) {
+    if (this.#lastStateCheckAt == query) {
       // If query is the same, return early.
-      return this._state;
+      return this.#state;
     }
-    this._lastStateCheckAt = query;
+    this.#lastStateCheckAt = query;
 
-    this._state = null;
+    this.#state = null;
     let subQuery = "";
     // Now we iterate over the query and decide the state character by
     // character.
@@ -241,20 +251,20 @@ SelectorAutocompleter.prototype = {
       // Calculate the state.
       subQuery = query.slice(0, i);
       let [secondLastChar, lastChar] = subQuery.slice(-2);
-      switch (this._state) {
+      switch (this.#state) {
         case null:
           // This will happen only in the first iteration of the for loop.
           lastChar = secondLastChar;
 
         case this.States.TAG: // eslint-disable-line
           if (lastChar === ".") {
-            this._state = this.States.CLASS;
+            this.#state = this.States.CLASS;
           } else if (lastChar === "#") {
-            this._state = this.States.ID;
+            this.#state = this.States.ID;
           } else if (lastChar === "[") {
-            this._state = this.States.ATTRIBUTE;
+            this.#state = this.States.ATTRIBUTE;
           } else {
-            this._state = this.States.TAG;
+            this.#state = this.States.TAG;
           }
           break;
 
@@ -263,13 +273,13 @@ SelectorAutocompleter.prototype = {
             // Checks whether the subQuery has atleast one [a-zA-Z] after the
             // '.'.
             if (lastChar === " " || lastChar === ">") {
-              this._state = this.States.TAG;
+              this.#state = this.States.TAG;
             } else if (lastChar === "#") {
-              this._state = this.States.ID;
+              this.#state = this.States.ID;
             } else if (lastChar === "[") {
-              this._state = this.States.ATTRIBUTE;
+              this.#state = this.States.ATTRIBUTE;
             } else {
-              this._state = this.States.CLASS;
+              this.#state = this.States.CLASS;
             }
           }
           break;
@@ -279,13 +289,13 @@ SelectorAutocompleter.prototype = {
             // Checks whether the subQuery has atleast one [a-zA-Z] after the
             // '#'.
             if (lastChar === " " || lastChar === ">") {
-              this._state = this.States.TAG;
+              this.#state = this.States.TAG;
             } else if (lastChar === ".") {
-              this._state = this.States.CLASS;
+              this.#state = this.States.CLASS;
             } else if (lastChar === "[") {
-              this._state = this.States.ATTRIBUTE;
+              this.#state = this.States.ATTRIBUTE;
             } else {
-              this._state = this.States.ID;
+              this.#state = this.States.ID;
             }
           }
           break;
@@ -294,43 +304,41 @@ SelectorAutocompleter.prototype = {
           if (subQuery.match(/[\[][^\]]+[\]]/) !== null) {
             // Checks whether the subQuery has at least one ']' after the '['.
             if (lastChar === " " || lastChar === ">") {
-              this._state = this.States.TAG;
+              this.#state = this.States.TAG;
             } else if (lastChar === ".") {
-              this._state = this.States.CLASS;
+              this.#state = this.States.CLASS;
             } else if (lastChar === "#") {
-              this._state = this.States.ID;
+              this.#state = this.States.ID;
             } else {
-              this._state = this.States.ATTRIBUTE;
+              this.#state = this.States.ATTRIBUTE;
             }
           }
           break;
       }
     }
-    return this._state;
-  },
-  /* eslint-enable complexity */
+    return this.#state;
+  }
 
   /**
    * Removes event listeners and cleans up references.
    */
-  destroy: function() {
+  destroy() {
     this.searchBox.removeEventListener("input", this.showSuggestions, true);
     this.searchBox.removeEventListener(
       "keypress",
-      this._onSearchKeypress,
+      this.#onSearchKeypress,
       true
     );
-    this.inspector.off("markupmutation", this._onMarkupMutation);
     this.searchPopup.destroy();
     this.searchPopup = null;
     this.searchBox = null;
     this.panelDoc = null;
-  },
+  }
 
   /**
    * Handles keypresses inside the input box.
    */
-  _onSearchKeypress: function(event) {
+  #onSearchKeypress = event => {
     const popup = this.searchPopup;
     switch (event.keyCode) {
       case KeyCodes.DOM_VK_RETURN:
@@ -344,7 +352,7 @@ SelectorAutocompleter.prototype = {
           // When tab is pressed with focus on searchbox and closed popup,
           // do not prevent the default to avoid a keyboard trap and move focus
           // to next/previous element.
-          this.emit("processing-done");
+          this.emitForTests("processing-done");
           return;
         }
         break;
@@ -367,7 +375,7 @@ SelectorAutocompleter.prototype = {
         if (popup.isOpen) {
           this.hidePopup();
         } else {
-          this.emit("processing-done");
+          this.emitForTests("processing-done");
           return;
         }
         break;
@@ -378,13 +386,13 @@ SelectorAutocompleter.prototype = {
 
     event.preventDefault();
     event.stopPropagation();
-    this.emit("processing-done");
-  },
+    this.emitForTests("processing-done");
+  };
 
   /**
    * Handles click events from the autocomplete popup.
    */
-  _onSearchPopupClick: function(event) {
+  #onSearchPopupClick = event => {
     const selectedItem = this.searchPopup.selectedItem;
     if (selectedItem) {
       this.searchBox.value = selectedItem.label;
@@ -393,16 +401,7 @@ SelectorAutocompleter.prototype = {
 
     event.preventDefault();
     event.stopPropagation();
-  },
-
-  /**
-   * Reset previous search results on markup-mutations to make sure we search
-   * again after nodes have been added/removed/changed.
-   */
-  _onMarkupMutation: function() {
-    this._searchResults = null;
-    this._lastSearched = null;
-  },
+  };
 
   /**
    * Populates the suggestions list and show the suggestion popup.
@@ -410,22 +409,22 @@ SelectorAutocompleter.prototype = {
    * @return {Promise} promise that will resolve when the autocomplete popup is fully
    * displayed or hidden.
    */
-  _showPopup: function(list, popupState) {
+  #showPopup(list, popupState) {
     let total = 0;
     const query = this.searchBox.value;
     const items = [];
 
     for (let [value, , state] of list) {
-      if (query.match(/[\s>+]$/)) {
-        // for cases like 'div ' or 'div >' or 'div+'
+      if (query.match(/[\s>+~]$/)) {
+        // for cases like 'div ', 'div >', 'div+' or 'div~'
         value = query + value;
-      } else if (query.match(/[\s>+][\.#a-zA-Z][^\s>+\.#\[]*$/)) {
+      } else if (query.match(/[\s>+~][\.#a-zA-Z][^\s>+~\.#\[]*$/)) {
         // for cases like 'div #a' or 'div .a' or 'div > d' and likewise
-        const lastPart = query.match(/[\s>+][\.#a-zA-Z][^\s>+\.#\[]*$/)[0];
+        const lastPart = query.match(/[\s>+~][\.#a-zA-Z][^\s>+~\.#\[]*$/)[0];
         value = query.slice(0, -1 * lastPart.length + 1) + value;
-      } else if (query.match(/[a-zA-Z][#\.][^#\.\s+>]*$/)) {
+      } else if (query.match(/[a-zA-Z][#\.][^#\.\s+>~]*$/)) {
         // for cases like 'div.class' or '#foo.bar' and likewise
-        const lastPart = query.match(/[a-zA-Z][#\.][^#\.\s+>]*$/)[0];
+        const lastPart = query.match(/[a-zA-Z][#\.][^#\.\s+>~]*$/)[0];
         value = query.slice(0, -1 * lastPart.length + 1) + value;
       } else if (query.match(/[a-zA-Z]*\[[^\]]*\][^\]]*/)) {
         // for cases like '[foo].bar' and likewise
@@ -466,23 +465,25 @@ SelectorAutocompleter.prototype = {
     }
 
     return this.hidePopup();
-  },
+  }
 
   /**
    * Hide the suggestion popup if necessary.
    */
-  hidePopup: function() {
+  hidePopup() {
     const onPopupClosed = this.searchPopup.once("popup-closed");
     this.searchPopup.hidePopup();
     return onPopupClosed;
-  },
+  }
 
   /**
    * Suggests classes,ids and tags based on the user input as user types in the
    * searchbox.
    */
-  showSuggestions: function() {
+  async showSuggestions() {
     let query = this.searchBox.value;
+    const originalQuery = this.searchBox.value;
+
     const state = this.state;
     let firstPart = "";
 
@@ -491,13 +492,20 @@ SelectorAutocompleter.prototype = {
       // suggest all nodes) or if it is an attribute selector (because
       // it would give a lot of useless results).
       this.hidePopup();
+      this.emitForTests("processing-done", { query: originalQuery });
       return;
     }
 
     if (state === this.States.TAG) {
-      // gets the tag that is being completed. For ex. 'div.foo > s' returns
-      // 's', 'di' returns 'di' and likewise.
-      firstPart = (query.match(/[\s>+]?([a-zA-Z]*)$/) || ["", query])[1];
+      // gets the tag that is being completed. For ex:
+      // - 'di' returns 'di'
+      // - 'div.foo s' returns 's'
+      // - 'div.foo > s' returns 's'
+      // - 'div.foo + s' returns 's'
+      // - 'div.foo ~ s' returns 's'
+      // - 'div.foo x-el_1' returns 'x-el_1'
+      const matches = query.match(/[\s>+~]?(?<tag>[a-zA-Z0-9_-]*)$/);
+      firstPart = matches.groups.tag;
       query = query.slice(0, query.length - firstPart.length);
     } else if (state === this.States.CLASS) {
       // gets the class that is being completed. For ex. '.foo.b' returns 'b'
@@ -514,37 +522,30 @@ SelectorAutocompleter.prototype = {
       query += "*";
     }
 
-    const suggestionsPromise = this.walker.getSuggestionsForQuery(
-      query,
-      firstPart,
-      state
-    );
-    this._lastQuery = suggestionsPromise.then(result => {
-      this.emit("processing-done");
-      if (result.query !== query) {
-        // This means that this response is for a previous request and the user
-        // as since typed something extra leading to a new request.
-        return promise.resolve(null);
-      }
+    let suggestions =
+      await this.inspector.commands.inspectorCommand.getSuggestionsForQuery(
+        query,
+        firstPart,
+        state
+      );
 
-      if (state === this.States.CLASS) {
-        firstPart = "." + firstPart;
-      } else if (state === this.States.ID) {
-        firstPart = "#" + firstPart;
-      }
+    if (state === this.States.CLASS) {
+      firstPart = "." + firstPart;
+    } else if (state === this.States.ID) {
+      firstPart = "#" + firstPart;
+    }
 
-      // If there is a single tag match and it's what the user typed, then
-      // don't need to show a popup.
-      if (
-        result.suggestions.length === 1 &&
-        result.suggestions[0][0] === firstPart
-      ) {
-        result.suggestions = [];
-      }
+    // If there is a single tag match and it's what the user typed, then
+    // don't need to show a popup.
+    if (suggestions.length === 1 && suggestions[0][0] === firstPart) {
+      suggestions = [];
+    }
 
-      // Wait for the autocomplete-popup to fire its popup-opened event, to make sure
-      // the autoSelect item has been selected.
-      return this._showPopup(result.suggestions, state);
-    });
-  },
-};
+    // Wait for the autocomplete-popup to fire its popup-opened event, to make sure
+    // the autoSelect item has been selected.
+    await this.#showPopup(suggestions, state);
+    this.emitForTests("processing-done", { query: originalQuery });
+  }
+}
+
+exports.SelectorAutocompleter = SelectorAutocompleter;
