@@ -6,9 +6,9 @@
 const {
   Component,
   createFactory,
-} = require("resource://devtools/client/shared/vendor/react.js");
+} = require("resource://devtools/client/shared/vendor/react.mjs");
 const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
-const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
 const {
   L10N,
 } = require("resource://devtools/client/netmonitor/src/utils/l10n.js");
@@ -58,9 +58,10 @@ const SearchBox = createFactory(
   require("resource://devtools/client/shared/components/SearchBox.js")
 );
 
-loader.lazyGetter(this, "MODE", function() {
-  return require("resource://devtools/client/shared/components/reps/index.js")
-    .MODE;
+loader.lazyGetter(this, "MODE", function () {
+  return ChromeUtils.importESModule(
+    "resource://devtools/client/shared/components/reps/index.mjs"
+  ).MODE;
 });
 
 const { div, input, label, span, h2 } = dom;
@@ -86,6 +87,8 @@ class ResponsePanel extends Component {
       targetSearchResult: PropTypes.object,
       connector: PropTypes.object.isRequired,
       showMessagesView: PropTypes.bool,
+      defaultRawResponse: PropTypes.bool,
+      setDefaultRawResponse: PropTypes.func,
     };
   }
 
@@ -94,14 +97,14 @@ class ResponsePanel extends Component {
 
     this.state = {
       filterText: "",
-      rawResponsePayloadDisplayed: !!props.targetSearchResult,
+      rawResponsePayloadDisplayed:
+        !!props.targetSearchResult || !!props.defaultRawResponse,
     };
 
     this.toggleRawResponsePayload = this.toggleRawResponsePayload.bind(this);
     this.renderCORSBlockedReason = this.renderCORSBlockedReason.bind(this);
-    this.renderRawResponsePayloadBtn = this.renderRawResponsePayloadBtn.bind(
-      this
-    );
+    this.renderRawResponsePayloadBtn =
+      this.renderRawResponsePayloadBtn.bind(this);
     this.renderJsonHtmlAndSource = this.renderJsonHtmlAndSource.bind(this);
     this.handleJSONResponse = this.handleJSONResponse.bind(this);
   }
@@ -229,7 +232,7 @@ class ResponsePanel extends Component {
       image: "",
       priority: PriorityLevels.PRIORITY_INFO_HIGH,
       type: "info",
-      eventCallback: e => {},
+      eventCallback: () => {},
       buttons: [
         {
           mdnUrl: getCORSErrorURL(blockedReason),
@@ -299,9 +302,8 @@ class ResponsePanel extends Component {
       // If raw response payload is not displayed render xssi info box if
       // there are stripped chars
       if (!rawResponsePayloadDisplayed) {
-        xssiStrippedCharsInfoBox = this.renderXssiStrippedCharsInfoBox(
-          strippedChars
-        );
+        xssiStrippedCharsInfoBox =
+          this.renderXssiStrippedCharsInfoBox(strippedChars);
       } else {
         xssiStrippedCharsInfoBox = null;
       }
@@ -343,13 +345,12 @@ class ResponsePanel extends Component {
     };
   }
 
-  renderRawResponsePayloadBtn(key, checked, onChange) {
+  renderRawResponsePayloadBtn(key, checked) {
     return [
       label(
         {
           key: `${key}RawResponsePayloadBtn`,
           className: "raw-data-toggle",
-          htmlFor: `raw-${key}-checkbox`,
           onClick: event => {
             // stop the header click event
             event.stopPropagation();
@@ -362,7 +363,12 @@ class ResponsePanel extends Component {
             id: `raw-${key}-checkbox`,
             checked,
             className: "devtools-checkbox-toggle",
-            onChange,
+            onChange: event => {
+              if (this.props.setDefaultRawResponse) {
+                this.props.setDefaultRawResponse(event.target.checked);
+              }
+              this.toggleRawResponsePayload();
+            },
             type: "checkbox",
           })
         )
@@ -396,7 +402,7 @@ class ResponsePanel extends Component {
       image: "",
       priority: PriorityLevels.PRIORITY_INFO_MEDIUM,
       type: "info",
-      eventCallback: e => {},
+      eventCallback: () => {},
       buttons: [],
     });
 
@@ -414,9 +420,8 @@ class ResponsePanel extends Component {
     const { filterText, rawResponsePayloadDisplayed } = this.state;
 
     // Display CORS blocked Reason info box
-    const CORSBlockedReasonDetails = this.renderCORSBlockedReason(
-      blockedReason
-    );
+    const CORSBlockedReasonDetails =
+      this.renderCORSBlockedReason(blockedReason);
 
     if (showMessagesView) {
       return MessagesView({ connector });
@@ -486,8 +491,7 @@ class ResponsePanel extends Component {
         hasFormattedDisplay &&
           this.renderRawResponsePayloadBtn(
             "response",
-            rawResponsePayloadDisplayed,
-            this.toggleRawResponsePayload
+            rawResponsePayloadDisplayed
           ),
       ]),
       xssiStrippedCharsInfoBox,
