@@ -11,8 +11,13 @@ const {
   getResponseHeader,
   getStartTime,
   ipToLong,
-} = require("./request-utils");
-const { RESPONSE_HEADERS } = require("../constants");
+} = require("resource://devtools/client/netmonitor/src/utils/request-utils.js");
+const {
+  RESPONSE_HEADERS,
+} = require("resource://devtools/client/netmonitor/src/constants.js");
+const {
+  getUrlBaseName,
+} = require("resource://devtools/client/netmonitor/src/utils/request-utils.js");
 
 /**
  * Predicates used when sorting items.
@@ -35,12 +40,17 @@ function compareValues(first, second) {
 }
 
 function waterfall(first, second) {
-  const result = compareValues(first.startedMillis, second.startedMillis);
+  const result = compareValues(first.startedMs, second.startedMs);
   return result || compareValues(first.id, second.id);
 }
 
+function priority(first, second) {
+  const result = compareValues(first.priority, second.priority);
+  return result || waterfall(first, second);
+}
+
 function status(first, second) {
-  const result = compareValues(first.status, second.status);
+  const result = compareValues(getStatusValue(first), getStatusValue(second));
   return result || waterfall(first, second);
 }
 
@@ -56,6 +66,13 @@ function file(first, second) {
   return result || waterfall(first, second);
 }
 
+function path(first, second) {
+  const firstUrl = first.urlDetails.path.toLowerCase();
+  const secondUrl = second.urlDetails.path.toLowerCase();
+  const result = compareValues(firstUrl, secondUrl);
+  return result || waterfall(first, second);
+}
+
 function url(first, second) {
   const firstUrl = first.url.toLowerCase();
   const secondUrl = second.url.toLowerCase();
@@ -107,10 +124,32 @@ function latency(first, second) {
 }
 
 function compareHeader(header, first, second) {
-  const result = compareValues(
-    getResponseHeader(first, header) || "",
-    getResponseHeader(second, header) || ""
-  );
+  const firstValue = getResponseHeader(first, header) || "";
+  const secondValue = getResponseHeader(second, header) || "";
+
+  let result;
+
+  switch (header) {
+    case "Content-Length": {
+      result = compareValues(
+        parseInt(firstValue, 10) || 0,
+        parseInt(secondValue, 10) || 0
+      );
+      break;
+    }
+    case "Last-Modified": {
+      result = compareValues(
+        new Date(firstValue).valueOf() || -1,
+        new Date(secondValue).valueOf() || -1
+      );
+      break;
+    }
+    default: {
+      result = compareValues(firstValue, secondValue);
+      break;
+    }
+  }
+
   return result || waterfall(first, second);
 }
 
@@ -143,6 +182,46 @@ function cause(first, second) {
   return result || waterfall(first, second);
 }
 
+function initiator(first, second) {
+  const firstCause = first.cause.type;
+  const secondCause = second.cause.type;
+
+  let firstInitiator = "";
+  let firstInitiatorLineNumber = 0;
+
+  if (first.cause.lastFrame) {
+    firstInitiator = getUrlBaseName(first.cause.lastFrame.filename);
+    firstInitiatorLineNumber = first.cause.lastFrame.lineNumber;
+  }
+
+  let secondInitiator = "";
+  let secondInitiatorLineNumber = 0;
+
+  if (second.cause.lastFrame) {
+    secondInitiator = getUrlBaseName(second.cause.lastFrame.filename);
+    secondInitiatorLineNumber = second.cause.lastFrame.lineNumber;
+  }
+
+  let result;
+  // if both initiators don't have a stack trace, compare their causes
+  if (!firstInitiator && !secondInitiator) {
+    result = compareValues(firstCause, secondCause);
+  } else if (!firstInitiator || !secondInitiator) {
+    // if one initiator doesn't have a stack trace but the other does, former should precede the latter
+    result = compareValues(firstInitiatorLineNumber, secondInitiatorLineNumber);
+  } else {
+    result = compareValues(firstInitiator, secondInitiator);
+    if (result === 0) {
+      result = compareValues(
+        firstInitiatorLineNumber,
+        secondInitiatorLineNumber
+      );
+    }
+  }
+
+  return result || waterfall(first, second);
+}
+
 function setCookies(first, second) {
   let { responseCookies: firstResponseCookies = { cookies: [] } } = first;
   let { responseCookies: secondResponseCookies = { cookies: [] } } = second;
@@ -175,8 +254,43 @@ function type(first, second) {
   return result || waterfall(first, second);
 }
 
+function getStatusValue(item) {
+  let value;
+  if (item.blockedReason) {
+    value = typeof item.blockedReason == "number" ? -item.blockedReason : -1000;
+  } else if (item.status == null) {
+    value = -2;
+  } else {
+    value = item.status;
+  }
+  return value;
+}
+
+function getTransferedSizeValue(item) {
+  let value;
+  if (item.blockedReason) {
+    // Also properly group/sort various blocked reasons.
+    value = typeof item.blockedReason == "number" ? -item.blockedReason : -1000;
+  } else if (item.fromCache || item.status === "304") {
+    value = -2;
+  } else if (item.fromServiceWorker) {
+    value = -3;
+  } else if (typeof item.transferredSize == "number") {
+    value = item.transferredSize;
+    if (item.isRacing && typeof item.isRacing == "boolean") {
+      value = -4;
+    }
+  } else if (item.transferredSize === null) {
+    value = -5;
+  }
+  return value;
+}
+
 function transferred(first, second) {
-  const result = compareValues(first.transferredSize, second.transferredSize);
+  const result = compareValues(
+    getTransferedSizeValue(first),
+    getTransferedSizeValue(second)
+  );
   return result || waterfall(first, second);
 }
 
@@ -190,12 +304,14 @@ const sorters = {
   method,
   domain,
   file,
+  path,
   protocol,
   scheme,
   cookies,
   setCookies,
   remoteip,
   cause,
+  initiator,
   type,
   transferred,
   contentSize,
@@ -206,5 +322,6 @@ const sorters = {
   latency,
   waterfall,
   url,
+  priority,
 };
 exports.Sorters = Object.assign(sorters, responseHeaders);
