@@ -7,10 +7,15 @@
  * Tests Curl Utils functionality.
  */
 
-const { Curl, CurlUtils } = require("devtools/client/shared/curl");
+const {
+  Curl,
+  CurlUtils,
+} = require("resource://devtools/client/shared/curl.js");
 
-add_task(async function() {
-  const { tab, monitor } = await initNetMonitor(CURL_UTILS_URL);
+add_task(async function () {
+  const { tab, monitor } = await initNetMonitor(HTTPS_CURL_UTILS_URL, {
+    requestCount: 1,
+  });
   info("Starting test... ");
 
   const { store, windowRequire, connector } = monitor.panelWin;
@@ -22,18 +27,23 @@ add_task(async function() {
 
   store.dispatch(Actions.batchEnable(false));
 
-  const wait = waitForNetworkEvents(monitor, 5);
-  await ContentTask.spawn(tab.linkedBrowser, SIMPLE_SJS, async function(url) {
-    content.wrappedJSObject.performRequests(url);
-  });
+  const wait = waitForNetworkEvents(monitor, 6);
+  await SpecialPowers.spawn(
+    tab.linkedBrowser,
+    [HTTPS_SIMPLE_SJS],
+    async function (url) {
+      content.wrappedJSObject.performRequests(url);
+    }
+  );
   await wait;
 
   const requests = {
-    get: getSortedRequests(store.getState()).get(0),
-    post: getSortedRequests(store.getState()).get(1),
-    patch: getSortedRequests(store.getState()).get(2),
-    multipart: getSortedRequests(store.getState()).get(3),
-    multipartForm: getSortedRequests(store.getState()).get(4),
+    get: getSortedRequests(store.getState())[0],
+    post: getSortedRequests(store.getState())[1],
+    postJson: getSortedRequests(store.getState())[2],
+    patch: getSortedRequests(store.getState())[3],
+    multipart: getSortedRequests(store.getState())[4],
+    multipartForm: getSortedRequests(store.getState())[5],
   };
 
   let data = await createCurlData(requests.get, getLongString, requestData);
@@ -49,6 +59,9 @@ add_task(async function() {
   testWritePostDataTextParams(data);
   testDataArgumentOnGeneratedCommand(data);
 
+  data = await createCurlData(requests.postJson, getLongString, requestData);
+  testDataEscapeOnGeneratedCommand(data);
+
   data = await createCurlData(requests.multipart, getLongString, requestData);
   testIsMultipartRequest(data);
   testGetMultipartBoundary(data);
@@ -86,7 +99,7 @@ function testIsMultipartRequest(data) {
 }
 
 function testFindHeader(data) {
-  const headers = data.headers;
+  const { headers } = data;
   const hostName = CurlUtils.findHeader(headers, "Host");
   const requestedWithLowerCased = CurlUtils.findHeader(
     headers,
@@ -108,7 +121,7 @@ function testFindHeader(data) {
 }
 
 function testMultiPartHeaders(data) {
-  const headers = data.headers;
+  const { headers } = data;
   const contentType = CurlUtils.findHeader(headers, "Content-Type");
 
   ok(
@@ -126,7 +139,7 @@ function testWritePostDataTextParams(data) {
   );
 }
 
-function testWriteEmptyPostDataTextParams(data) {
+function testWriteEmptyPostDataTextParams() {
   const params = CurlUtils.writePostDataTextParams(null);
   is(params, "", "Should return a empty string when no parameters provided");
 }
@@ -134,8 +147,25 @@ function testWriteEmptyPostDataTextParams(data) {
 function testDataArgumentOnGeneratedCommand(data) {
   const curlCommand = Curl.generateCommand(data);
   ok(
-    curlCommand.includes("--data"),
-    "Should return a curl command with --data"
+    curlCommand.includes("--data-raw"),
+    "Should return a curl command with --data-raw"
+  );
+}
+
+function testDataEscapeOnGeneratedCommand(data) {
+  const paramsWin = `--data-raw "{""param1"":""value1"",""param2"":""value2""}"`;
+  const paramsPosix = `--data-raw '{"param1":"value1","param2":"value2"}'`;
+
+  let curlCommand = Curl.generateCommand(data, "WINNT");
+  ok(
+    curlCommand.includes(paramsWin),
+    "Should return a curl command with --data-raw escaped for Windows systems"
+  );
+
+  curlCommand = Curl.generateCommand(data, "Linux");
+  ok(
+    curlCommand.includes(paramsPosix),
+    "Should return a curl command with --data-raw escaped for Posix systems"
   );
 }
 
@@ -213,7 +243,7 @@ function testGetHeadersFromMultipartText(data) {
   const headers = CurlUtils.getHeadersFromMultipartText(data.postDataText);
 
   ok(Array.isArray(headers), "Should return an array.");
-  ok(headers.length > 0, "There should exist at least one request header.");
+  ok(!!headers.length, "There should exist at least one request header.");
   is(
     headers[0].name,
     "Content-Type",
@@ -239,7 +269,7 @@ function testEscapeStringPosix() {
   const escapeChar = "'!ls:q:gs|ls|;ping 8.8.8.8;|";
   is(
     CurlUtils.escapeStringPosix(escapeChar),
-    "$'\\'\\041ls:q:gs|ls|;ping 8.8.8.8;|'",
+    "$'\\'\\041ls:q:gs^|ls^|;ping 8.8.8.8;^|'",
     "'!' should be escaped."
   );
 
@@ -265,6 +295,14 @@ function testEscapeStringPosix() {
     "$'\\xc3\\xa6 \\xc3\\xb8 \\xc3\\xbc \\xc3\\x9f \\xc3\\xb6 \\xc3\\xa9'",
     "Character codes outside of the decimal range 32 - 126 should be escaped."
   );
+
+  // Assert that ampersands are correctly escaped in case its tried to run on Windows
+  const evilCommand = `query=evil\n\ncmd & calc.exe\n\n`;
+  is(
+    CurlUtils.escapeStringPosix(evilCommand),
+    "$'query=evil\\n\\ncmd ^& calc.exe\\n\\n'",
+    "The evil command is escaped properly"
+  );
 }
 
 function testEscapeStringWin() {
@@ -282,10 +320,10 @@ function testEscapeStringWin() {
     "Double quotes should be escaped."
   );
 
-  const percentSigns = "%AppData%";
+  const percentSigns = "%TEMP% %@foo% %2XX% %_XX% %?XX%";
   is(
     CurlUtils.escapeStringWin(percentSigns),
-    '""%"AppData"%""',
+    '"^%^TEMP^% ^%^@foo^% ^%^2XX^% ^%^_XX^% ^%?XX^%"',
     "Percent signs should be escaped."
   );
 
@@ -296,12 +334,33 @@ function testEscapeStringWin() {
     "Backslashes should be escaped."
   );
 
-  const newLines = "line1\r\nline2\r\nline3";
+  const newLines = "line1\r\nline2\r\rline3\n\nline4";
   is(
     CurlUtils.escapeStringWin(newLines),
-    '"line1"^\u000d\u000A\u000d\u000A"line2"^\u000d\u000A\u000d\u000A"line3"',
+    '"line1"^\r\n\r\n"line2"^\r\n\r\n""^\r\n\r\n"line3"^\r\n\r\n""^\r\n\r\n"line4"',
     "Newlines should be escaped."
   );
+
+  const dollarSignCommand = "$(calc.exe)";
+  is(
+    CurlUtils.escapeStringWin(dollarSignCommand),
+    '"\\$(calc.exe)"',
+    "Dollar sign should be escaped."
+  );
+
+  const tickSignCommand = "`$(calc.exe)";
+  is(
+    CurlUtils.escapeStringWin(tickSignCommand),
+    '"\\`\\$(calc.exe)"',
+    "Both the tick and dollar signs should be escaped."
+  );
+
+  const evilCommand = `query=evil\r\rcmd" /c timeout /t 3 & calc.exe\r\r`;
+  is(
+    CurlUtils.escapeStringWin(evilCommand),
+    '"query=evil"^\r\n\r\n""^\r\n\r\n"cmd"" /c timeout /t 3 & calc.exe"^\r\n\r\n""^\r\n\r\n""',
+    "The evil command is escaped properly"
+  );
 }
 
 async function createCurlData(selected, getLongString, requestData) {
@@ -320,7 +379,7 @@ async function createCurlData(selected, getLongString, requestData) {
   // Fetch header values.
   for (const { name, value } of requestHeaders.headers) {
     const text = await getLongString(value);
-    data.headers.push({ name: name, value: text });
+    data.headers.push({ name, value: text });
   }
 
   const requestPostData = await requestData(id, "requestPostData");
