@@ -238,6 +238,23 @@ async function disableCacheAndReload(toolbox, waitForLoad) {
   }
 }
 
+async function enableCacheAndReload(toolbox, waitForLoad) {
+  // Disable the cache for any toolbox that it is opened from this point on.
+  Services.prefs.setBoolPref("devtools.cache.disabled", false);
+
+  await toolbox.commands.targetConfigurationCommand.updateConfiguration({
+    cacheDisabled: false,
+  });
+
+  // If the page which is reloaded is not found, this will likely cause
+  // reloadTopLevelTarget to not return so let not wait for it.
+  if (waitForLoad) {
+    await toolbox.commands.targetCommand.reloadTopLevelTarget();
+  } else {
+    toolbox.commands.targetCommand.reloadTopLevelTarget();
+  }
+}
+
 /**
  * Wait for 2 markers during document load.
  */
@@ -383,6 +400,22 @@ function initNetMonitor(
       await disableCacheAndReload(toolbox, waitForLoad);
       await Promise.all(allComplete);
       await clearNetworkEvents(monitor);
+    } else if (Services.prefs.getBoolPref("devtools.cache.disabled")) {
+      info("Enabling cache and reloading page.");
+
+      const allComplete = [];
+      allComplete.push(
+        waitForNetworkEvents(monitor, requestCount, {
+          expectedEventTimings,
+        })
+      );
+
+      if (waitForLoad) {
+        allComplete.push(waitForTimelineMarkers(monitor));
+      }
+      await enableCacheAndReload(toolbox, waitForLoad);
+      await Promise.all(allComplete);
+      await clearNetworkEvents(monitor);
     }
 
     return { tab, monitor, toolbox, privateWindow };
@@ -1726,3 +1759,15 @@ function getCurrentVisibleColumns(monitor) {
   // to return name.
   return visibleColumns.map(([name]) => name);
 }
+
+function findRequestByInitiator(document, initiator) {
+  for (const request of document.querySelectorAll(".request-list-item")) {
+    if (
+      request.querySelector(".requests-list-initiator").getAttribute("title") ==
+      initiator
+    ) {
+      return request;
+    }
+  }
+  return null;
+}
