@@ -6,9 +6,9 @@
 
 const {
   Component,
-} = require("resource://devtools/client/shared/vendor/react.js");
+} = require("resource://devtools/client/shared/vendor/react.mjs");
 const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
-const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
 const {
   getUnicodeUrl,
   getUnicodeUrlPath,
@@ -110,8 +110,7 @@ class Frame extends Component {
     this._locationChanged = this._locationChanged.bind(this);
   }
 
-  // FIXME: https://bugzilla.mozilla.org/show_bug.cgi?id=1774507
-  UNSAFE_componentWillMount() {
+  componentDidMount() {
     if (this.props.sourceMapURLService) {
       const location = savedFrameToLocation(this.props.frame);
       // Many things that make use of this component either:
@@ -121,10 +120,11 @@ class Frame extends Component {
       //
       // and due to that, we need to use subscribeByLocation in order to
       // handle both cases with an without an ID.
-      this.unsubscribeSourceMapURLService = this.props.sourceMapURLService.subscribeByLocation(
-        location,
-        this._locationChanged
-      );
+      this.unsubscribeSourceMapURLService =
+        this.props.sourceMapURLService.subscribeByLocation(
+          location,
+          this._locationChanged
+        );
     }
   }
 
@@ -258,10 +258,13 @@ class Frame extends Component {
       return {
         ...sourceElConfig,
         onClick: e => {
+          // We always need to prevent the default behavior of <a> link
           e.preventDefault();
-          e.stopPropagation();
+          if (onClick) {
+            e.stopPropagation();
 
-          onClick(generatedLocation);
+            onClick(generatedLocation);
+          }
         },
         href: source,
         draggable: false,
@@ -322,6 +325,12 @@ class Frame extends Component {
     let displaySource = showFullSourceUrl ? unicodeLong : unicodeShort;
     if (originalLocation) {
       displaySource = getSourceMappedFile(displaySource);
+
+      // In case of pretty-printed HTML file, we would only get the formatted suffix; replace
+      // it with the full URL instead
+      if (showEmptyPathAsHost && displaySource == ":formatted") {
+        displaySource = host + displaySource;
+      }
     } else if (
       showEmptyPathAsHost &&
       (displaySource === "" || displaySource === "/")
