@@ -4,10 +4,26 @@
 
 "use strict";
 
-const React = require("resource://devtools/client/shared/vendor/react.js");
+const React = require("resource://devtools/client/shared/vendor/react.mjs");
 const { Component, createFactory } = React;
 const dom = require("resource://devtools/client/shared/vendor/react-dom-factories.js");
-const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.js");
+const PropTypes = require("resource://devtools/client/shared/vendor/react-prop-types.mjs");
+
+// Localized strings for (devtools/client/locales/en-US/components.properties)
+loader.lazyGetter(this, "L10N_COMPONENTS", function () {
+  const { LocalizationHelper } = require("resource://devtools/shared/l10n.js");
+  return new LocalizationHelper(
+    "devtools/client/locales/components.properties"
+  );
+});
+
+loader.lazyGetter(this, "EXPAND_LABEL", function () {
+  return L10N_COMPONENTS.getStr("treeNode.expandButtonTitle");
+});
+
+loader.lazyGetter(this, "COLLAPSE_LABEL", function () {
+  return L10N_COMPONENTS.getStr("treeNode.collapseButtonTitle");
+});
 
 // depth
 const AUTO_EXPAND_DEPTH = 0;
@@ -35,19 +51,22 @@ class ArrowExpander extends Component {
     };
   }
 
-  shouldComponentUpdate(nextProps, nextState) {
+  shouldComponentUpdate(nextProps) {
     return this.props.expanded !== nextProps.expanded;
   }
 
   render() {
     const { expanded } = this.props;
 
-    const classNames = ["arrow"];
+    const classNames = ["theme-twisty"];
+    const title = expanded ? COLLAPSE_LABEL : EXPAND_LABEL;
+
     if (expanded) {
-      classNames.push("expanded");
+      classNames.push("open");
     }
     return dom.button({
       className: classNames.join(" "),
+      title,
     });
   }
 }
@@ -241,7 +260,7 @@ const TreeNodeFactory = createFactory(TreeNode);
 function oncePerAnimationFrame(fn, { getDocument }) {
   let animationId = null;
   let argsToPass = null;
-  return function(...args) {
+  return function (...args) {
     argsToPass = args;
     if (animationId !== null) {
       return;
@@ -306,8 +325,6 @@ function oncePerAnimationFrame(fn, { getDocument }) {
  *
  *       render() {
  *         return Tree({
- *           itemHeight: 20, // px
- *
  *           getRoots: () => [this.props.root],
  *
  *           getParent: item => item.parent,
@@ -538,22 +555,19 @@ class Tree extends Component {
   }
 
   // FIXME: https://bugzilla.mozilla.org/show_bug.cgi?id=1774507
-  UNSAFE_componentWillReceiveProps(nextProps) {
+  UNSAFE_componentWillReceiveProps() {
     this._autoExpand();
   }
 
-  componentDidUpdate(prevProps, prevState) {
+  componentDidUpdate(prevProps) {
     if (this.props.focused && prevProps.focused !== this.props.focused) {
       this._scrollNodeIntoView(this.props.focused);
     }
   }
 
   _autoExpand() {
-    const {
-      autoExpandDepth,
-      autoExpandNodeChildrenLimit,
-      initiallyExpanded,
-    } = this.props;
+    const { autoExpandDepth, autoExpandNodeChildrenLimit, initiallyExpanded } =
+      this.props;
 
     if (!autoExpandDepth && !initiallyExpanded) {
       return;
