@@ -32,7 +32,8 @@ if (DEBUG_ALLOCATIONS) {
     useDistinctSystemPrincipalLoader,
     releaseDistinctSystemPrincipalLoader,
   } = ChromeUtils.importESModule(
-    "resource://devtools/shared/loader/DistinctSystemPrincipalLoader.sys.mjs"
+    "resource://devtools/shared/loader/DistinctSystemPrincipalLoader.sys.mjs",
+    { global: "shared" }
   );
   const requester = {};
   const loader = useDistinctSystemPrincipalLoader(requester);
@@ -54,9 +55,76 @@ if (DEBUG_ALLOCATIONS) {
   });
 }
 
+// When DEBUG_STEP environment variable is set,
+// automatically start a tracer which will log all line being executed
+// in the running test (and nothing else) and also pause its execution
+// for the given amount of milliseconds.
+//
+// Be careful that these pause have significant side effect.
+// This will pause the test script event loop and allow running the other
+// tasks queued in the parent process's main thread event loop queue.
+//
+// Passing any non-number value, like `DEBUG_STEP=true` will still
+// log the executed lines without any pause, and without this side effect.
+//
+// For now, the tracer can only work once per thread.
+// So when using this feature you will not be able to use the JS tracer
+// in any other way on parent process's main thread.
+const DEBUG_STEP = Services.env.get("DEBUG_STEP");
+if (DEBUG_STEP) {
+  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
+  // as it instantiates custom Debugger API instances and has to be running in a distinct
+  // compartments from DevTools and system scopes (JSMs, XPCOM,...)
+  const {
+    useDistinctSystemPrincipalLoader,
+    releaseDistinctSystemPrincipalLoader,
+  } = ChromeUtils.importESModule(
+    "resource://devtools/shared/loader/DistinctSystemPrincipalLoader.sys.mjs",
+    { global: "shared" }
+  );
+  const requester = {};
+  const loader = useDistinctSystemPrincipalLoader(requester);
+
+  const stepper = loader.require(
+    "resource://devtools/shared/test-helpers/test-stepper.js"
+  );
+  stepper.start(globalThis, gTestPath, DEBUG_STEP);
+  registerCleanupFunction(() => {
+    stepper.stop();
+    releaseDistinctSystemPrincipalLoader(requester);
+  });
+}
+
+const DEBUG_TRACE_LINE = Services.env.get("DEBUG_TRACE_LINE");
+if (DEBUG_TRACE_LINE) {
+  // Use a custom loader with `invisibleToDebugger` flag for the allocation tracker
+  // as it instantiates custom Debugger API instances and has to be running in a distinct
+  // compartments from DevTools and system scopes (ESMs, XPCOM,...)
+  const {
+    useDistinctSystemPrincipalLoader,
+    releaseDistinctSystemPrincipalLoader,
+  } = ChromeUtils.importESModule(
+    "resource://devtools/shared/loader/DistinctSystemPrincipalLoader.sys.mjs"
+  );
+  const requester = {};
+  const loader = useDistinctSystemPrincipalLoader(requester);
+
+  const lineTracer = loader.require(
+    "resource://devtools/shared/test-helpers/test-line-tracer.js"
+  );
+  lineTracer.start(globalThis, gTestPath, DEBUG_TRACE_LINE);
+  registerCleanupFunction(() => {
+    lineTracer.stop();
+    releaseDistinctSystemPrincipalLoader(requester);
+  });
+}
+
 const { loader, require } = ChromeUtils.importESModule(
   "resource://devtools/shared/loader/Loader.sys.mjs"
 );
+const { sinon } = ChromeUtils.importESModule(
+  "resource://testing-common/Sinon.sys.mjs"
+);
 
 // When loaded from xpcshell test, this file is loaded via xpcshell.ini's head property
 // and so it loaded first before anything else and isn't having access to Services global.
@@ -73,6 +141,8 @@ const DevToolsUtils = require("resource://devtools/shared/DevToolsUtils.js");
 
 const KeyShortcuts = require("resource://devtools/client/shared/key-shortcuts.js");
 
+const { LocalizationHelper } = require("resource://devtools/shared/l10n.js");
+
 loader.lazyRequireGetter(
   this,
   "ResponsiveUIManager",
@@ -162,7 +232,7 @@ if (isMochitest) {
 
 var EXPECTED_DTU_ASSERT_FAILURE_COUNT = 0;
 
-registerCleanupFunction(function() {
+registerCleanupFunction(function () {
   if (
     DevToolsUtils.assertionFailureCount !== EXPECTED_DTU_ASSERT_FAILURE_COUNT
   ) {
@@ -209,26 +279,29 @@ Services.prefs.setBoolPref("devtools.inspector.three-pane-enabled", true);
 // requests occuring after a process is created/destroyed. See Bug 1620983.
 Services.prefs.setBoolPref("dom.ipc.processPrelaunch.enabled", false);
 
-// Disable this preference to capture async stacks across all locations during
-// DevTools mochitests. Async stacks provide very valuable information to debug
-// intermittents, but come with a performance overhead, which is why they are
-// only captured in Debuggees by default.
+// On some Linux platforms, prefers-reduced-motion is enabled, which would
+// trigger the notification to be displayed in the toolbox. Dismiss the message
+// by default.
 Services.prefs.setBoolPref(
-  "javascript.options.asyncstack_capture_debuggee_only",
-  false
+  "devtools.inspector.simple-highlighters.message-dismissed",
+  true
 );
 
+// Enable dumping scope variables when a test failure occurs.
+Services.prefs.setBoolPref("devtools.testing.testScopes", true);
+
 registerCleanupFunction(() => {
   Services.prefs.clearUserPref("devtools.dump.emit");
   Services.prefs.clearUserPref("devtools.inspector.three-pane-enabled");
   Services.prefs.clearUserPref("dom.ipc.processPrelaunch.enabled");
   Services.prefs.clearUserPref("devtools.toolbox.host");
   Services.prefs.clearUserPref("devtools.toolbox.previousHost");
-  Services.prefs.clearUserPref("devtools.toolbox.splitconsoleEnabled");
+  Services.prefs.clearUserPref("devtools.toolbox.splitconsole.open");
   Services.prefs.clearUserPref("devtools.toolbox.splitconsoleHeight");
   Services.prefs.clearUserPref(
-    "javascript.options.asyncstack_capture_debuggee_only"
+    "devtools.inspector.simple-highlighters.message-dismissed"
   );
+  Services.prefs.clearUserPref("devtools.testing.testScopes");
 });
 
 var {
@@ -564,7 +637,7 @@ async function navigateTo(
   if (uri === browser.currentURI.spec) {
     gBrowser.reloadTab(gBrowser.getTabForBrowser(browser));
   } else {
-    BrowserTestUtils.loadURIString(browser, uri);
+    BrowserTestUtils.startLoadingURIString(browser, uri);
   }
 
   if (waitForLoad) {
@@ -620,7 +693,7 @@ async function watchForDevToolsReload(
     waitForLoad,
   });
 
-  return async function() {
+  return async function () {
     info("Wait for the toolbox to reload");
     await waitForToolboxReload();
 
@@ -640,11 +713,11 @@ async function _watchForToolboxReload(
 ) {
   const tab = gBrowser.getTabForBrowser(browser);
 
-  const toolbox = await gDevTools.getToolboxForTab(tab);
+  const toolbox = gDevTools.getToolboxForTab(tab);
 
   if (!toolbox) {
     // No toolbox to wait for
-    return function() {};
+    return function () {};
   }
 
   const waitForCurrentPanelReload = watchForCurrentPanelReload(toolbox);
@@ -657,7 +730,7 @@ async function _watchForToolboxReload(
     browser
   );
 
-  return async function() {
+  return async function () {
     const isTargetSwitching = checkTargetSwitching();
 
     info(`Waiting for toolbox commands to be reloaded…`);
@@ -687,7 +760,7 @@ async function _watchForResponsiveReload(
 
   if (!ui) {
     // No responsive UI to wait for
-    return function() {};
+    return function () {};
   }
 
   const onResponsiveTargetSwitch = ui.once("responsive-ui-target-switch-done");
@@ -700,7 +773,7 @@ async function _watchForResponsiveReload(
     browser
   );
 
-  return async function() {
+  return async function () {
     const isTargetSwitching = checkTargetSwitching();
 
     info(`Waiting for responsive ui commands to be reloaded…`);
@@ -751,7 +824,7 @@ function watchForLoadedPanelsReload(toolbox) {
     waitForPanels.push(_watchForPanelReload(toolbox, id));
   }
 
-  return function() {
+  return function () {
     return Promise.all(
       waitForPanels.map(async watchPanel => {
         // Wait for all panels to be reloaded.
@@ -772,7 +845,7 @@ function _watchForPanelReload(toolbox, toolId) {
     const onUpdated = panel.once("inspector-updated");
     const onReloaded = panel.once("reloaded");
 
-    return async function() {
+    return async function () {
       info("Waiting for markup view to load after navigation.");
       await markuploaded;
 
@@ -789,7 +862,7 @@ function _watchForPanelReload(toolbox, toolId) {
     ["netmonitor", "accessibility", "webconsole", "jsdebugger"].includes(toolId)
   ) {
     const onReloaded = panel.once("reloaded");
-    return async function() {
+    return async function () {
       info(`Waiting for ${toolId} updates after page reload`);
       await onReloaded;
     };
@@ -830,18 +903,18 @@ async function watchForCommandsReload(
     ? "dom-complete"
     : "dom-loading";
 
-  const {
-    onResource: onTopLevelDomEvent,
-  } = await commands.resourceCommand.waitForNextResource(
-    commands.resourceCommand.TYPES.DOCUMENT_EVENT,
-    {
-      ignoreExistingResources: true,
-      predicate: resource =>
-        resource.targetFront.isTopLevel && resource.name === documentEventName,
-    }
-  );
+  const { onResource: onTopLevelDomEvent } =
+    await commands.resourceCommand.waitForNextResource(
+      commands.resourceCommand.TYPES.DOCUMENT_EVENT,
+      {
+        ignoreExistingResources: true,
+        predicate: resource =>
+          resource.targetFront.isTopLevel &&
+          resource.name === documentEventName,
+      }
+    );
 
-  return async function(isTargetSwitching) {
+  return async function (isTargetSwitching) {
     if (typeof isTargetSwitching === "undefined") {
       throw new Error("isTargetSwitching was not provided to the wait method");
     }
@@ -878,7 +951,7 @@ async function watchForTargetSwitching(commands, browser) {
   const targetFollowsWindowLifecycle =
     commands.targetCommand.targetFront.targetForm.followWindowGlobalLifeCycle;
 
-  return function() {
+  return function () {
     // Compare the PIDs (and not the toolbox's targets) as PIDs are updated also immediately,
     // while target may be updated slightly later.
     const switchedProcess =
@@ -935,14 +1008,13 @@ function isEveryFrameTargetEnabled() {
  */
 async function openInspectorForURL(url, hostType) {
   const tab = await addTab(url);
-  const { inspector, toolbox, highlighterTestFront } = await openInspector(
-    hostType
-  );
+  const { inspector, toolbox, highlighterTestFront } =
+    await openInspector(hostType);
   return { tab, inspector, toolbox, highlighterTestFront };
 }
 
-async function getActiveInspector() {
-  const toolbox = await gDevTools.getToolboxForTab(gBrowser.selectedTab);
+function getActiveInspector() {
+  const toolbox = gDevTools.getToolboxForTab(gBrowser.selectedTab);
   return toolbox.getPanel("inspector");
 }
 
@@ -955,9 +1027,7 @@ async function getActiveInspector() {
  *        Optional window where to fire the key event
  */
 function synthesizeKeyShortcut(key, target) {
-  // parseElectronKey requires any window, just to access `KeyboardEvent`
-  const window = Services.appShell.hiddenDOMWindow;
-  const shortcut = KeyShortcuts.parseElectronKey(window, key);
+  const shortcut = KeyShortcuts.parseElectronKey(key);
   const keyEvent = {
     altKey: shortcut.alt,
     ctrlKey: shortcut.ctrl,
@@ -1012,10 +1082,19 @@ function wait(ms) {
  * @param number maxTries [optional]
  *        How many times the predicate is invoked before timing out.
  *        Can be set globally for a test via `waitFor.overrideMaxTriesForTestFile = someNumber;`.
+ * @param boolean expectTimeout [optional]
+ *        Whether the helper is expected to reach the timeout or not. Consider
+ *        using waitForTimeout instead of passing this to true.
  * @return object
  *         A promise that is resolved with the result of the condition.
  */
-async function waitFor(condition, message = "", interval = 10, maxTries = 500) {
+async function waitFor(
+  condition,
+  message = "",
+  interval = 10,
+  maxTries = 500,
+  expectTimeout = false
+) {
   // Update interval & maxTries if overrides are defined on the waitFor object.
   interval =
     typeof waitFor.overrideIntervalForTestFile !== "undefined"
@@ -1033,13 +1112,46 @@ async function waitFor(condition, message = "", interval = 10, maxTries = 500) {
       interval,
       maxTries
     );
+    if (expectTimeout) {
+      // If we expected a timeout, fail the test here.
+      const errorMessage = `Expected timeout in waitFor(): ${message} \nUnexpected condition: ${condition} \n`;
+      ok(false, errorMessage);
+    }
     return value;
   } catch (e) {
+    if (expectTimeout) {
+      // If we expected a timeout, simply return null, the consumer should not
+      // need any return value.
+      return null;
+    }
+
+    // If we didn't expect a timeout, fail the test here.
     const errorMessage = `Failed waitFor(): ${message} \nFailed condition: ${condition} \nException Message: ${e}`;
+    // Use both assert ok(false) and throw.
+    // Assert captures the correct frame to log variables with dump-scope.js.
+    // Error will make sure the test stops.
+    ok(false, errorMessage);
     throw new Error(errorMessage);
   }
 }
 
+/**
+ * Similar to @see waitFor defined above but expect the predicate to never
+ * be satisfied and therefore to timeout.
+ *
+ * Arguments are identical to `waitFor` but the default values for interval and
+ * maxTries are more conservative since this method expects a timeout.
+ *
+ */
+async function waitForTimeout(
+  condition,
+  message = "",
+  interval = 100,
+  maxTries = 10
+) {
+  return waitFor(condition, message, interval, maxTries, true);
+}
+
 /**
  * Wait for eventName on target to be delivered a number of times.
  *
@@ -1161,10 +1273,8 @@ function loadHelperScript(filePath) {
 async function openToolboxForTab(tab, toolId, hostType) {
   info("Opening the toolbox");
 
-  let toolbox;
-
   // Check if the toolbox is already loaded.
-  toolbox = await gDevTools.getToolboxForTab(tab);
+  let toolbox = gDevTools.getToolboxForTab(tab);
   if (toolbox) {
     if (!toolId || (toolId && toolbox.getPanel(toolId))) {
       info("Toolbox is already opened");
@@ -1236,7 +1346,7 @@ function waitUntil(predicate, interval = 10) {
     return Promise.resolve(true);
   }
   return new Promise(resolve => {
-    setTimeout(function() {
+    setTimeout(function () {
       waitUntil(predicate, interval).then(() => resolve(true));
     }, interval);
   });
@@ -1371,8 +1481,8 @@ function isWindows() {
  * @returns {HttpServer}
  */
 function createTestHTTPServer() {
-  const { HttpServer } = ChromeUtils.import(
-    "resource://testing-common/httpd.js"
+  const { HttpServer } = ChromeUtils.importESModule(
+    "resource://testing-common/httpd.sys.mjs"
   );
   const server = new HttpServer();
 
@@ -1457,7 +1567,7 @@ function getCurrentTestFilePath() {
 async function unregisterAllServiceWorkers(client) {
   info("Wait until all workers have a valid registrationFront");
   let workers;
-  await asyncWaitUntil(async function() {
+  await asyncWaitUntil(async function () {
     workers = await client.mainRoot.listAllWorkers();
     const allWorkersRegistered = workers.service.every(
       worker => !!worker.registrationFront
@@ -1520,7 +1630,7 @@ async function waitUntilScreenshot({ isWindowPrivate = false } = {}) {
   );
   const list = await Downloads.getList(Downloads.ALL);
 
-  return new Promise(function(resolve) {
+  return new Promise(function (resolve) {
     const view = {
       onDownloadAdded: async download => {
         await download.whenSucceeded();
@@ -1830,12 +1940,15 @@ async function safeSynthesizeMouseEventAtCenterInContentPage(
  * @returns {Promise}
  */
 function scrollContentPageNodeIntoView(browsingContext, selector) {
-  return SpecialPowers.spawn(browsingContext, [selector], function(
-    innerSelector
-  ) {
-    const node = content.wrappedJSObject.document.querySelector(innerSelector);
-    node.scrollIntoView();
-  });
+  return SpecialPowers.spawn(
+    browsingContext,
+    [selector],
+    function (innerSelector) {
+      const node =
+        content.wrappedJSObject.document.querySelector(innerSelector);
+      node.scrollIntoView();
+    }
+  );
 }
 
 /**
@@ -1860,16 +1973,15 @@ function setContentPageZoomLevel(zoomLevel) {
  *         is received.
  */
 async function waitForNextTopLevelDomCompleteResource(commands) {
-  const {
-    onResource: onDomCompleteResource,
-  } = await commands.resourceCommand.waitForNextResource(
-    commands.resourceCommand.TYPES.DOCUMENT_EVENT,
-    {
-      ignoreExistingResources: true,
-      predicate: resource =>
-        resource.name === "dom-complete" && resource.targetFront.isTopLevel,
-    }
-  );
+  const { onResource: onDomCompleteResource } =
+    await commands.resourceCommand.waitForNextResource(
+      commands.resourceCommand.TYPES.DOCUMENT_EVENT,
+      {
+        ignoreExistingResources: true,
+        predicate: resource =>
+          resource.name === "dom-complete" && resource.targetFront.isTopLevel,
+      }
+    );
   return { onDomCompleteResource };
 }
 
@@ -2077,7 +2189,7 @@ function createVersionizedHttpTestServer(testFolderName) {
     let fetchResponse;
 
     if (request.queryString) {
-      const url = `${URL_ROOT}${testFolderName}/v${currentVersion}${request.path}.${request.queryString}`;
+      const url = `${URL_ROOT_SSL}${testFolderName}/v${currentVersion}${request.path}.${request.queryString}`;
       try {
         fetchResponse = await fetch(url);
         // Log this only if the request succeed
@@ -2089,7 +2201,7 @@ function createVersionizedHttpTestServer(testFolderName) {
     }
 
     if (!fetchResponse) {
-      const url = `${URL_ROOT}${testFolderName}/v${currentVersion}${request.path}`;
+      const url = `${URL_ROOT_SSL}${testFolderName}/v${currentVersion}${request.path}`;
       info(`[test-http-server] serving: ${url}`);
       fetchResponse = await fetch(url);
     }
@@ -2122,3 +2234,305 @@ function createVersionizedHttpTestServer(testFolderName) {
     },
   };
 }
+
+/**
+ * Fake clicking a link and return the URL we would have navigated to.
+ * This function should be used to check external links since we can't access
+ * network in tests.
+ * This can also be used to test that a click will not be fired.
+ *
+ * @param ElementNode element
+ *        The <a> element we want to simulate click on.
+ * @returns Promise
+ *          A Promise that is resolved when the link click simulation occured or
+ *          when the click is not dispatched.
+ *          The promise resolves with an object that holds the following properties
+ *          - link: url of the link or null(if event not fired)
+ *          - where: "tab" if tab is active or "tabshifted" if tab is inactive
+ *            or null(if event not fired)
+ */
+function simulateLinkClick(element) {
+  const browserWindow = Services.wm.getMostRecentWindow(
+    gDevTools.chromeWindowType
+  );
+
+  const onOpenLink = new Promise(resolve => {
+    const openLinkIn = (link, where) => resolve({ link, where });
+    sinon.replace(browserWindow, "openTrustedLinkIn", openLinkIn);
+    sinon.replace(browserWindow, "openWebLinkIn", openLinkIn);
+  });
+
+  element.click();
+
+  // Declare a timeout Promise that we can use to make sure spied methods were not called.
+  const onTimeout = new Promise(function (resolve) {
+    setTimeout(() => {
+      resolve({ link: null, where: null });
+    }, 1000);
+  });
+
+  const raceResult = Promise.race([onOpenLink, onTimeout]);
+  sinon.restore();
+  return raceResult;
+}
+
+/**
+ * Since the MDN data is updated frequently, it might happen that the properties used in
+ * this test are not in the dataset anymore/now have URLs.
+ * This function will return properties in the dataset that don't have MDN url so you
+ * can easily find a replacement.
+ */
+function logCssCompatDataPropertiesWithoutMDNUrl() {
+  const cssPropertiesCompatData = require("resource://devtools/shared/compatibility/dataset/css-properties.json");
+
+  function walk(node) {
+    for (const propertyName in node) {
+      const property = node[propertyName];
+      if (property.__compat) {
+        if (!property.__compat.mdn_url) {
+          dump(
+            `"${propertyName}" - MDN URL: ${
+              property.__compat.mdn_url || "❌"
+            } - Spec URL: ${property.__compat.spec_url || "❌"}\n`
+          );
+        }
+      } else if (typeof property == "object") {
+        walk(property);
+      }
+    }
+  }
+  walk(cssPropertiesCompatData);
+}
+
+/**
+ * Craft a CssProperties instance without involving RDP for tests
+ * manually spawning OutputParser, CssCompleter, Editor...
+ *
+ * Otherwise this should instead be fetched from CssPropertiesFront.
+ *
+ * @return {CssProperties}
+ */
+function getClientCssProperties() {
+  const {
+    generateCssProperties,
+  } = require("resource://devtools/server/actors/css-properties.js");
+  const {
+    CssProperties,
+    normalizeCssData,
+  } = require("resource://devtools/client/fronts/css-properties.js");
+  return new CssProperties(
+    normalizeCssData({ properties: generateCssProperties(document) })
+  );
+}
+
+/**
+ * Helper method to stop a Service Worker promptly.
+ *
+ * @param {String} workerUrl
+ *        Absolute Worker URL to stop.
+ */
+async function stopServiceWorker(workerUrl) {
+  info(`Stop Service Worker: ${workerUrl}\n`);
+
+  // Help the SW to be immediately destroyed after unregistering it.
+  Services.prefs.setIntPref("dom.serviceWorkers.idle_timeout", 0);
+
+  const swm = Cc["@mozilla.org/serviceworkers/manager;1"].getService(
+    Ci.nsIServiceWorkerManager
+  );
+  // Unfortunately we can't use swm.getRegistrationByPrincipal, as it requires a "scope", which doesn't seem to be the worker URL.
+  // So let's use getAllRegistrations to find the nsIServiceWorkerInfo in order to:
+  // - retrieve its active worker,
+  // - call attach+detachDebugger,
+  // - reset the idle timeout.
+  // This way, the unregister instruction is immediate, thanks to the 0 dom.serviceWorkers.idle_timeout we set at the beginning of the function
+  const registrations = swm.getAllRegistrations();
+  let matchedInfo;
+  for (let i = 0; i < registrations.length; i++) {
+    const info = registrations.queryElementAt(
+      i,
+      Ci.nsIServiceWorkerRegistrationInfo
+    );
+    // Lookup for an exact URL match.
+    if (info.scriptSpec === workerUrl) {
+      matchedInfo = info;
+      break;
+    }
+  }
+  ok(!!matchedInfo, "Found the service worker info");
+
+  info("Wait for the worker to be active");
+  await waitFor(() => matchedInfo.activeWorker, "Wait for the SW to be active");
+
+  // We need to attach+detach the debugger in order to reset the idle timeout.
+  // Otherwise the worker would still be waiting for a previously registered timeout
+  // which would be the 0ms one we set by tweaking the preference.
+  function resetWorkerTimeout(worker) {
+    worker.attachDebugger();
+    worker.detachDebugger();
+  }
+  resetWorkerTimeout(matchedInfo.activeWorker);
+  // Also reset all the other possible worker instances
+  if (matchedInfo.evaluatingWorker) {
+    resetWorkerTimeout(matchedInfo.evaluatingWorker);
+  }
+  if (matchedInfo.installingWorker) {
+    resetWorkerTimeout(matchedInfo.installingWorker);
+  }
+  if (matchedInfo.waitingWorker) {
+    resetWorkerTimeout(matchedInfo.waitingWorker);
+  }
+  // Reset this preference in order to ensure other SW are not immediately destroyed.
+  Services.prefs.clearUserPref("dom.serviceWorkers.idle_timeout");
+
+  // Spin the event loop to ensure the worker had time to really be shut down.
+  await wait(0);
+
+  return matchedInfo;
+}
+
+/**
+ * Helper method to stop and unregister a Service Worker promptly.
+ *
+ * @param {String} workerUrl
+ *        Absolute Worker URL to unregister.
+ */
+async function unregisterServiceWorker(workerUrl) {
+  const swInfo = await stopServiceWorker(workerUrl);
+
+  info(`Unregister Service Worker: ${workerUrl}\n`);
+  // Now call unregister on that worker so that it can be destroyed immediately
+  const swm = Cc["@mozilla.org/serviceworkers/manager;1"].getService(
+    Ci.nsIServiceWorkerManager
+  );
+  const unregisterSuccess = await new Promise(resolve => {
+    swm.unregister(
+      swInfo.principal,
+      {
+        unregisterSucceeded(success) {
+          resolve(success);
+        },
+      },
+      swInfo.scope
+    );
+  });
+  ok(unregisterSuccess, "Service worker successfully unregistered");
+}
+
+/**
+ * Toggle the JavavaScript tracer via its toolbox toolbar button.
+ */
+async function toggleJsTracer(toolbox) {
+  const { tracerCommand } = toolbox.commands;
+  const { isTracingEnabled } = tracerCommand;
+  const { logMethod, traceOnNextInteraction, traceOnNextLoad } =
+    toolbox.commands.tracerCommand.getTracingOptions();
+
+  // When the tracer is waiting for user interaction or page load, it won't be made active
+  // right away. The test should manually wait for its activation.
+  const shouldWaitForToggle = !traceOnNextInteraction && !traceOnNextLoad;
+  let onTracingToggled;
+  if (shouldWaitForToggle) {
+    onTracingToggled = new Promise(resolve => {
+      tracerCommand.on("toggle", async function listener() {
+        // Ignore the event, if we are still in the same state as before the click
+        if (tracerCommand.isTracingActive == isTracingEnabled) {
+          return;
+        }
+        tracerCommand.off("toggle", listener);
+        resolve();
+      });
+    });
+  }
+
+  const toolbarButton = toolbox.doc.getElementById("command-button-jstracer");
+  toolbarButton.click();
+
+  if (shouldWaitForToggle) {
+    info("Waiting for the tracer to be active");
+    await onTracingToggled;
+  }
+
+  const {
+    TRACER_LOG_METHODS,
+  } = require("resource://devtools/shared/specs/tracer.js");
+  if (logMethod != TRACER_LOG_METHODS.CONSOLE) {
+    return;
+  }
+
+  // We were tracing and just requested to stop it.
+  // Wait for the stop message to appear in the console before clearing its content.
+  // This simplifies writting tests toggling the tracer ON multiple times and checking
+  // for the display of traces in the console.
+  if (isTracingEnabled) {
+    const { hud } = await toolbox.getPanel("webconsole");
+    info("Wait for tracing to be disabled");
+    await waitFor(() =>
+      [...hud.ui.outputNode.querySelectorAll(".message")].some(msg =>
+        msg.textContent.includes("Stopped tracing")
+      )
+    );
+
+    hud.ui.clearOutput();
+    await waitFor(
+      () => hud.ui.outputNode.querySelectorAll(".message").length === 0
+    );
+  } else {
+    // We are enabling the tracing to the console, and the console may not be opened just yet.
+    const { hud } = await toolbox.getPanelWhenReady("webconsole");
+    if (!traceOnNextInteraction && !traceOnNextLoad) {
+      await waitFor(() =>
+        [...hud.ui.outputNode.querySelectorAll(".message")].some(msg =>
+          msg.textContent.includes("Started tracing to Web Console")
+        )
+      );
+    }
+  }
+}
+
+/**
+ * Retrieve the context menu element corresponding to the provided id, for the
+ * provided netmonitor instance.
+ * @param {Object} monitor
+ *        The network monitor object
+ * @param {String} id
+ *        The id of the context menu item
+ */
+function getNetmonitorContextMenuItem(monitor, id) {
+  const Menu = require("resource://devtools/client/framework/menu.js");
+  return Menu.getMenuElementById(id, monitor.panelWin.document);
+}
+
+/*
+ * Selects and clicks the context menu item of the netmonitor, it should
+ * also wait for the popup to close.
+ * @param {Object} monitor
+ *        The network monitor object
+ * @param {String} id
+ *        The id of the context menu item
+ */
+async function selectNetmonitorContextMenuItem(monitor, id) {
+  const contextMenuItem = getNetmonitorContextMenuItem(monitor, id);
+
+  const popup = contextMenuItem.parentNode;
+  await _maybeOpenAncestorMenu(contextMenuItem);
+  const hidden = BrowserTestUtils.waitForEvent(popup, "popuphidden");
+  popup.activateItem(contextMenuItem);
+  await hidden;
+}
+
+async function _maybeOpenAncestorMenu(menuItem) {
+  const parentPopup = menuItem.parentNode;
+  if (parentPopup.state == "open") {
+    return;
+  }
+  const shown = BrowserTestUtils.waitForEvent(parentPopup, "popupshown");
+  if (parentPopup.state == "showing") {
+    await shown;
+    return;
+  }
+  const parentMenu = parentPopup.parentNode;
+  await _maybeOpenAncestorMenu(parentMenu);
+  parentMenu.openMenu(true);
+  await shown;
+}
