@@ -3,27 +3,24 @@
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 "use strict";
 
-const protocol = require("devtools/shared/protocol");
-const { ActorClassWithSpec, Actor } = protocol;
-const { perfSpec } = require("devtools/shared/specs/perf");
-const { Ci } = require("chrome");
-const Services = require("Services");
+const { Actor } = require("resource://devtools/shared/protocol.js");
+const { perfSpec } = require("resource://devtools/shared/specs/perf.js");
 
-loader.lazyImporter(
+loader.lazyRequireGetter(
   this,
-  "PrivateBrowsingUtils",
-  "resource://gre/modules/PrivateBrowsingUtils.jsm"
+  "RecordingUtils",
+  "resource://devtools/shared/performance-new/recording-utils.js"
 );
 
 // Some platforms are built without the Gecko Profiler.
 const IS_SUPPORTED_PLATFORM = "nsIProfiler" in Ci;
 
 /**
- * The PerfActor wraps the Gecko Profiler interface
+ * The PerfActor wraps the Gecko Profiler interface (aka Services.profiler).
  */
-exports.PerfActor = ActorClassWithSpec(perfSpec, {
-  initialize(conn) {
-    Actor.prototype.initialize.call(this, conn);
+exports.PerfActor = class PerfActor extends Actor {
+  constructor(conn) {
+    super(conn, perfSpec);
 
     // Only setup the observers on a supported platform.
     if (IS_SUPPORTED_PLATFORM) {
@@ -32,27 +29,18 @@ exports.PerfActor = ActorClassWithSpec(perfSpec, {
       };
       Services.obs.addObserver(this._observer, "profiler-started");
       Services.obs.addObserver(this._observer, "profiler-stopped");
-      Services.obs.addObserver(
-        this._observer,
-        "chrome-document-global-created"
-      );
-      Services.obs.addObserver(this._observer, "last-pb-context-exited");
     }
-  },
+  }
 
   destroy() {
+    super.destroy();
+
     if (!IS_SUPPORTED_PLATFORM) {
       return;
     }
     Services.obs.removeObserver(this._observer, "profiler-started");
     Services.obs.removeObserver(this._observer, "profiler-stopped");
-    Services.obs.removeObserver(
-      this._observer,
-      "chrome-document-global-created"
-    );
-    Services.obs.removeObserver(this._observer, "last-pb-context-exited");
-    Actor.prototype.destroy.call(this);
-  },
+  }
 
   startProfiler(options) {
     if (!IS_SUPPORTED_PLATFORM) {
@@ -63,18 +51,17 @@ exports.PerfActor = ActorClassWithSpec(perfSpec, {
     // to be tweaked or made configurable as needed.
     const settings = {
       entries: options.entries || 1000000,
-      // Window length should be Infinite if nothing's been passed.
-      // options.duration is supported for `perfActorVersion > 0`.
       duration: options.duration || 0,
       interval: options.interval || 1,
       features: options.features || [
         "js",
         "stackwalk",
+        "cpu",
         "responsiveness",
-        "threads",
-        "leaf",
+        "memory",
       ],
       threads: options.threads || ["GeckoMain", "Compositor"],
+      activeTabID: RecordingUtils.getActiveBrowserID(),
     };
 
     try {
@@ -84,6 +71,7 @@ exports.PerfActor = ActorClassWithSpec(perfSpec, {
         settings.interval,
         settings.features,
         settings.threads,
+        settings.activeTabID,
         settings.duration
       );
     } catch (e) {
@@ -92,15 +80,20 @@ exports.PerfActor = ActorClassWithSpec(perfSpec, {
     }
 
     return true;
-  },
+  }
 
   stopProfilerAndDiscardProfile() {
     if (!IS_SUPPORTED_PLATFORM) {
-      return;
+      return null;
     }
-    Services.profiler.StopProfiler();
-  },
+    return Services.profiler.StopProfiler();
+  }
 
+  /**
+   * @type {string} debugPath
+   * @type {string} breakpadId
+   * @returns {Promise<[number[], number[], number[]]>}
+   */
   async getSymbolTable(debugPath, breakpadId) {
     const [addr, index, buffer] = await Services.profiler.getSymbolTable(
       debugPath,
@@ -110,78 +103,87 @@ exports.PerfActor = ActorClassWithSpec(perfSpec, {
     // these typed arrays to plain JS arrays of numbers now.
     // Our return value type is declared as "array:array:number".
     return [Array.from(addr), Array.from(index), Array.from(buffer)];
-  },
+  }
 
   async getProfileAndStopProfiler() {
     if (!IS_SUPPORTED_PLATFORM) {
       return null;
     }
 
+    // Pause profiler before we collect the profile, so that we don't capture
+    // more samples while the parent process or android threads wait for subprocess profiles.
+    Services.profiler.Pause();
+
     let profile;
     try {
       // Attempt to pull out the data.
       profile = await Services.profiler.getProfileDataAsync();
 
-      // Stop and discard the buffers.
-      Services.profiler.StopProfiler();
+      if (Object.keys(profile).length === 0) {
+        console.error(
+          "An empty object was received from getProfileDataAsync.getProfileDataAsync(), " +
+            "meaning that a profile could not successfully be serialized and captured."
+        );
+        profile = null;
+      }
     } catch (e) {
-      // If there was any kind of error, bailout with no profile.
-      return null;
+      // Explicitly set the profile to null if there as an error.
+      profile = null;
+      console.error(`There was an error fetching a profile`, e);
     }
 
-    // Gecko Profiler errors can return an empty object, return null for this case
-    // as well.
-    if (Object.keys(profile).length === 0) {
-      return null;
-    }
+    // Stop and discard the buffers.
+    Services.profiler.StopProfiler();
+
+    // Returns a profile when successful, and null when there is an error.
     return profile;
-  },
+  }
 
   isActive() {
     if (!IS_SUPPORTED_PLATFORM) {
       return false;
     }
     return Services.profiler.IsActive();
-  },
+  }
 
   isSupportedPlatform() {
     return IS_SUPPORTED_PLATFORM;
-  },
-
-  isLockedForPrivateBrowsing() {
-    if (!IS_SUPPORTED_PLATFORM) {
-      return false;
-    }
-    return !Services.profiler.CanProfile();
-  },
+  }
 
   /**
-   * Watch for events that happen within the browser. These can affect the current
-   * availability and state of the Gecko Profiler.
+   * Watch for events that happen within the browser. These can affect the
+   * current availability and state of the Gecko Profiler.
    */
   _observe(subject, topic, _data) {
+    // Note! If emitting new events make sure and update the list of bridged
+    // events in the perf actor.
     switch (topic) {
-      case "chrome-document-global-created":
-        if (PrivateBrowsingUtils.isWindowPrivate(subject)) {
-          this.emit("profile-locked-by-private-browsing");
-        }
-        break;
-      case "last-pb-context-exited":
-        this.emit("profile-unlocked-from-private-browsing");
-        break;
-      case "profiler-started":
+      case "profiler-started": {
         const param = subject.QueryInterface(Ci.nsIProfilerStartParams);
         this.emit(
           topic,
           param.entries,
           param.interval,
           param.features,
-          param.duration
+          param.duration,
+          param.activeTabID
         );
         break;
+      }
       case "profiler-stopped":
         this.emit(topic);
         break;
     }
-  },
-});
+  }
+
+  /**
+   * Lists the supported features of the profiler for the current browser.
+   * @returns {string[]}
+   */
+  getSupportedFeatures() {
+    if (!IS_SUPPORTED_PLATFORM) {
+      return [];
+    }
+    return Services.profiler.GetFeatures();
+  }
+};
