@@ -1,13 +1,9 @@
-/* -*- indent-tabs-mode: nil; js-indent-level: 2 -*- */
-/* vim: set ts=2 et sw=2 tw=80: */
 /* This Source Code Form is subject to the terms of the Mozilla Public
  * License, v. 2.0. If a copy of the MPL was not distributed with this
  * file, You can obtain one at http://mozilla.org/MPL/2.0/. */
 
 "use strict";
 
-const InspectorUtils = require("InspectorUtils");
-
 const MAX_DATA_URL_LENGTH = 40;
 /**
  * Provide access to the style information in a page.
@@ -19,33 +15,19 @@ const MAX_DATA_URL_LENGTH = 40;
  * @constructor
  */
 
-const Services = require("Services");
-
-loader.lazyImporter(
-  this,
-  "findCssSelector",
-  "resource://gre/modules/css-selector.js"
-);
-loader.lazyImporter(
-  this,
-  "getCssPath",
-  "resource://gre/modules/css-selector.js"
-);
-loader.lazyImporter(this, "getXPath", "resource://gre/modules/css-selector.js");
 loader.lazyRequireGetter(
   this,
-  "getCSSLexer",
-  "devtools/shared/css/lexer",
+  "InspectorCSSParserWrapper",
+  "resource://devtools/shared/css/lexer.js",
   true
 );
 loader.lazyRequireGetter(
   this,
   "getTabPrefs",
-  "devtools/shared/indentation",
+  "resource://devtools/shared/indentation.js",
   true
 );
-
-const { LocalizationHelper } = require("devtools/shared/l10n");
+const { LocalizationHelper } = require("resource://devtools/shared/l10n.js");
 const styleInspectorL10N = new LocalizationHelper(
   "devtools/shared/locales/styleinspector.properties"
 );
@@ -77,23 +59,40 @@ exports.STATUS = {
 };
 
 /**
- * Mapping of CSSRule type value to CSSRule type name.
- * @see https://developer.mozilla.org/en-US/docs/Web/API/CSSRule
+ * Mapping of CSS at-Rule className to CSSRule type name.
  */
-exports.CSSRuleTypeName = {
-  1: "", // Regular CSS style rule has no name
-  3: "@import",
-  4: "@media",
-  5: "@font-face",
-  6: "@page",
-  7: "@keyframes",
-  8: "@keyframe",
-  10: "@namespace",
-  11: "@counter-style",
-  12: "@supports",
-  13: "@document",
-  14: "@font-feature-values",
-  15: "@viewport",
+exports.CSSAtRuleClassNameType = {
+  CSSContainerRule: "container",
+  CSSCounterStyleRule: "counter-style",
+  CSSDocumentRule: "document",
+  CSSFontFaceRule: "font-face",
+  CSSFontFeatureValuesRule: "font-feature-values",
+  CSSImportRule: "import",
+  CSSKeyframeRule: "keyframe",
+  CSSKeyframesRule: "keyframes",
+  CSSLayerBlockRule: "layer",
+  CSSMediaRule: "media",
+  CSSNamespaceRule: "namespace",
+  CSSPageRule: "page",
+  CSSScopeRule: "scope",
+  CSSStartingStyleRule: "starting-style",
+  CSSSupportsRule: "supports",
+};
+
+/**
+ * Get Rule type as human-readable string (ex: "@media", "@container", …)
+ *
+ * @param {CSSRule} cssRule
+ * @returns {String}
+ */
+exports.getCSSAtRuleTypeName = function (cssRule) {
+  const ruleClassName = ChromeUtils.getClassName(cssRule);
+  const atRuleTypeName = exports.CSSAtRuleClassNameType[ruleClassName];
+  if (atRuleTypeName) {
+    return "@" + atRuleTypeName;
+  }
+
+  return "";
 };
 
 /**
@@ -104,6 +103,8 @@ exports.CSSRuleTypeName = {
  * @returns {String} A localized version of the given key.
  */
 exports.l10n = name => styleInspectorL10N.getStr(name);
+exports.l10nFormatStr = (name, ...args) =>
+  styleInspectorL10N.getFormatStr(name, ...args);
 
 /**
  * Is the given property sheet an author stylesheet?
@@ -112,7 +113,7 @@ exports.l10n = name => styleInspectorL10N.getStr(name);
  * @return {boolean} true if the given stylesheet is an author stylesheet,
  * false otherwise.
  */
-exports.isAuthorStylesheet = function(sheet) {
+exports.isAuthorStylesheet = function (sheet) {
   return sheet.parsingMode === "author";
 };
 
@@ -123,7 +124,7 @@ exports.isAuthorStylesheet = function(sheet) {
  * @return {boolean} true if the given stylesheet is a user stylesheet,
  * false otherwise.
  */
-exports.isUserStylesheet = function(sheet) {
+exports.isUserStylesheet = function (sheet) {
   return sheet.parsingMode === "user";
 };
 
@@ -134,7 +135,7 @@ exports.isUserStylesheet = function(sheet) {
  * @return {boolean} true if the given stylesheet is a agent stylesheet,
  * false otherwise.
  */
-exports.isAgentStylesheet = function(sheet) {
+exports.isAgentStylesheet = function (sheet) {
   return sheet.parsingMode === "agent";
 };
 
@@ -143,38 +144,66 @@ exports.isAgentStylesheet = function(sheet) {
  *
  * @param {CSSStyleSheet} sheet the DOM object for the style sheet.
  */
-exports.shortSource = function(sheet) {
-  // Use a string like "inline" if there is no source href
-  if (!sheet || !sheet.href) {
+exports.shortSource = function (sheet) {
+  if (!sheet) {
     return exports.l10n("rule.sourceInline");
   }
 
+  if (!sheet.href) {
+    return exports.l10n(
+      sheet.constructed ? "rule.sourceConstructed" : "rule.sourceInline"
+    );
+  }
+
+  let name = sheet.href;
+
   // If the sheet is a data URL, return a trimmed version of it.
   const dataUrl = sheet.href.trim().match(/^data:.*?,((?:.|\r|\n)*)$/);
   if (dataUrl) {
-    return dataUrl[1].length > MAX_DATA_URL_LENGTH
-      ? `${dataUrl[1].substr(0, MAX_DATA_URL_LENGTH - 1)}…`
-      : dataUrl[1];
+    name =
+      dataUrl[1].length > MAX_DATA_URL_LENGTH
+        ? `${dataUrl[1].substr(0, MAX_DATA_URL_LENGTH - 1)}…`
+        : dataUrl[1];
+  } else {
+    // We try, in turn, the filename, filePath, query string, whole thing
+    const url = URL.parse(sheet.href);
+    if (url) {
+      if (url.pathname) {
+        const index = url.pathname.lastIndexOf("/");
+        if (index !== -1 && index < url.pathname.length) {
+          name = url.pathname.slice(index + 1);
+        } else {
+          name = url.pathname;
+        }
+      } else if (url.query) {
+        name = url.query;
+      }
+    } // else some UA-provided stylesheets are not valid URLs.
   }
 
-  // We try, in turn, the filename, filePath, query string, whole thing
-  let url = {};
   try {
-    url = new URL(sheet.href);
-  } catch (ex) {
-    // Some UA-provided stylesheets are not valid URLs.
+    name = decodeURIComponent(name);
+  } catch (e) {
+    // This may still fail if the URL contains invalid % numbers (for ex)
   }
 
-  if (url.pathname) {
-    const index = url.pathname.lastIndexOf("/");
-    if (index !== -1 && index < url.pathname.length) {
-      return url.pathname.slice(index + 1);
-    }
-    return url.pathname;
+  return name;
+};
+
+/**
+ * Return the style sheet's source, handling element, inline and constructed stylesheets.
+ *
+ * @param {CSSStyleSheet} sheet the DOM object for the style sheet.
+ */
+exports.longSource = function (sheet) {
+  if (!sheet) {
+    return exports.l10n("rule.sourceInline");
   }
 
-  if (url.query) {
-    return url.query;
+  if (!sheet.href) {
+    return exports.l10n(
+      sheet.constructed ? "rule.sourceConstructed" : "rule.sourceInline"
+    );
   }
 
   return sheet.href;
@@ -211,6 +240,7 @@ function getLineCountInComments(text) {
  *         The CSS source to prettify.
  * @param  {Number} ruleCount
  *         The number of CSS rules expected in the CSS source.
+ *         Set to null to force the text to be pretty-printed.
  *
  * @return {Object}
  *         Object with the prettified source and source mappings.
@@ -220,7 +250,7 @@ function getLineCountInComments(text) {
  *                              // between the original source and prettified source
  *          }
  */
-/* eslint-disable complexity */
+// eslint-disable-next-line complexity
 function prettifyCSS(text, ruleCount) {
   if (prettifyCSS.LINE_SEPARATOR == null) {
     const os = Services.appinfo.OS;
@@ -231,10 +261,7 @@ function prettifyCSS(text, ruleCount) {
   // before and after). Remove those first. Don't do anything there aren't any.
   const trimmed = text.trim();
   if (trimmed.startsWith("<!--")) {
-    text = trimmed
-      .replace(/^<!--/, "")
-      .replace(/-->$/, "")
-      .trim();
+    text = trimmed.replace(/^<!--/, "").replace(/-->$/, "").trim();
   }
 
   const originalText = text;
@@ -261,7 +288,7 @@ function prettifyCSS(text, ruleCount) {
   // minified file.
   let indent = "";
   let indentLevel = 0;
-  const tokens = getCSSLexer(text);
+  const lexer = new InspectorCSSParserWrapper(text);
   // List of mappings of token positions from original source to prettified source.
   const mappings = [];
   // Line and column offsets used to shift the token positions after prettyfication.
@@ -278,15 +305,15 @@ function prettifyCSS(text, ruleCount) {
   // seen.  This function also updates |pushbackToken|.
   const readUntilSignificantToken = () => {
     while (true) {
-      const token = tokens.nextToken();
-      if (!token || token.tokenType !== "whitespace") {
+      const token = lexer.nextToken();
+      if (!token || token.tokenType !== "WhiteSpace") {
         pushbackToken = token;
         return token;
       }
       // Saw whitespace.  Before committing to it, check the next
       // token.
-      const nextToken = tokens.nextToken();
-      if (!nextToken || nextToken.tokenType !== "comment") {
+      const nextToken = lexer.nextToken();
+      if (!nextToken || nextToken.tokenType !== "Comment") {
         pushbackToken = nextToken;
         return token;
       }
@@ -326,15 +353,15 @@ function prettifyCSS(text, ruleCount) {
         token = pushbackToken;
         pushbackToken = undefined;
       } else {
-        token = tokens.nextToken();
+        token = lexer.nextToken();
       }
       if (!token) {
         endIndex = text.length;
         break;
       }
 
-      const line = tokens.lineNumber;
-      const column = tokens.columnNumber;
+      const line = lexer.lineNumber;
+      const column = lexer.columnNumber;
       mappings.push({
         original: {
           line,
@@ -348,17 +375,17 @@ function prettifyCSS(text, ruleCount) {
       // Shift the column offset for the next token by the current token's length.
       columnOffset += token.endOffset - token.startOffset;
 
-      if (token.tokenType === "at") {
+      if (token.tokenType === "AtKeyword") {
         isInAtRuleDefinition = true;
       }
 
       // A "}" symbol must be inserted later, to deal with indentation
       // and newline.
-      if (token.tokenType === "symbol" && token.text === "}") {
+      if (token.tokenType === "CloseCurlyBracket") {
         isInSelector = true;
         isCloseBrace = true;
         break;
-      } else if (token.tokenType === "symbol" && token.text === "{") {
+      } else if (token.tokenType === "CurlyBracketBlock") {
         if (isInAtRuleDefinition) {
           isInAtRuleDefinition = false;
         } else {
@@ -367,7 +394,7 @@ function prettifyCSS(text, ruleCount) {
         break;
       }
 
-      if (token.tokenType !== "whitespace") {
+      if (token.tokenType !== "WhiteSpace") {
         anyNonWS = true;
       }
 
@@ -376,24 +403,30 @@ function prettifyCSS(text, ruleCount) {
       }
       endIndex = token.endOffset;
 
-      if (token.tokenType === "symbol" && token.text === ";") {
+      if (token.tokenType === "Semicolon") {
         break;
       }
 
       if (
-        token.tokenType === "symbol" &&
-        token.text === "," &&
+        token.tokenType === "Comma" &&
         isInSelector &&
         !isInAtRuleDefinition
       ) {
         break;
       }
 
-      lastWasWS = token.tokenType === "whitespace";
+      lastWasWS = token.tokenType === "WhiteSpace";
     }
     return token;
   };
 
+  // Get preference of the user regarding what to use for indentation,
+  // spaces or tabs.
+  const tabPrefs = getTabPrefs();
+  const baseIndentString = tabPrefs.indentWithTabs
+    ? TAB_CHARS
+    : SPACE_CHARS.repeat(tabPrefs.indentUnit);
+
   while (true) {
     // Set the initial state.
     startIndex = undefined;
@@ -419,20 +452,16 @@ function prettifyCSS(text, ruleCount) {
       }
     }
 
-    // Get preference of the user regarding what to use for indentation,
-    // spaces or tabs.
-    const tabPrefs = getTabPrefs();
-
     if (isCloseBrace) {
       // Even if the stylesheet contains extra closing braces, the indent level should
       // remain > 0.
       indentLevel = Math.max(0, indentLevel - 1);
+      indent = baseIndentString.repeat(indentLevel);
 
+      // FIXME: This is incorrect and should be fixed in Bug 1839297
       if (tabPrefs.indentWithTabs) {
-        indent = TAB_CHARS.repeat(indentLevel);
         indentOffset = 4 * indentLevel;
       } else {
-        indent = SPACE_CHARS.repeat(indentLevel);
         indentOffset = 1 * indentLevel;
       }
       result = result + indent + "}";
@@ -442,17 +471,20 @@ function prettifyCSS(text, ruleCount) {
       break;
     }
 
-    if (token.tokenType === "symbol" && token.text === "{") {
+    if (token.tokenType === "CurlyBracketBlock") {
       if (!lastWasWS) {
         result += " ";
         columnOffset++;
       }
       result += "{";
+      indentLevel++;
+      indent = baseIndentString.repeat(indentLevel);
+      indentOffset = indent.length;
+
+      // FIXME: This is incorrect and should be fixed in Bug 1839297
       if (tabPrefs.indentWithTabs) {
-        indent = TAB_CHARS.repeat(++indentLevel);
         indentOffset = 4 * indentLevel;
       } else {
-        indent = SPACE_CHARS.repeat(++indentLevel);
         indentOffset = 1 * indentLevel;
       }
     }
@@ -465,9 +497,10 @@ function prettifyCSS(text, ruleCount) {
     // Here we ignore the case where whitespace appears at the end of
     // the text.
     if (
+      ruleCount !== null &&
       pushbackToken &&
       token &&
-      token.tokenType === "whitespace" &&
+      token.tokenType === "WhiteSpace" &&
       /\n/g.test(text.substring(token.startOffset, token.endOffset))
     ) {
       return { result: originalText, mappings: [] };
@@ -488,32 +521,9 @@ function prettifyCSS(text, ruleCount) {
 
   return { result, mappings };
 }
-/* eslint-enable complexity */
 
 exports.prettifyCSS = prettifyCSS;
 
-/**
- * Find a unique CSS selector for a given element
- * @returns a string such that ele.ownerDocument.querySelector(reply) === ele
- * and ele.ownerDocument.querySelectorAll(reply).length === 1
- */
-exports.findCssSelector = findCssSelector;
-
-/**
- * Get the full CSS path for a given element.
- * @returns a string that can be used as a CSS selector for the element. It might not
- * match the element uniquely. It does however, represent the full path from the root
- * node to the element.
- */
-exports.getCssPath = getCssPath;
-
-/**
- * Get the xpath for a given element.
- * @param {DomNode} ele
- * @returns a string that can be used as an XPath to find the element uniquely.
- */
-exports.getXPath = getXPath;
-
 /**
  * Given a node, check to see if it is a ::marker, ::before, or ::after element.
  * If so, return the node that is accessible from within the document
@@ -522,36 +532,310 @@ exports.getXPath = getXPath;
  *
  * @returns {Object}
  *            - {DOMNode} node The non-anonymous node
- *            - {string} pseudo One of ':marker', ':before', ':after', or null.
+ *            - {string} pseudo One of '::marker', '::before', '::after', or null.
  */
 function getBindingElementAndPseudo(node) {
   let bindingElement = node;
   let pseudo = null;
   if (node.nodeName == "_moz_generated_content_marker") {
     bindingElement = node.parentNode;
-    pseudo = ":marker";
+    pseudo = "::marker";
   } else if (node.nodeName == "_moz_generated_content_before") {
     bindingElement = node.parentNode;
-    pseudo = ":before";
+    pseudo = "::before";
   } else if (node.nodeName == "_moz_generated_content_after") {
     bindingElement = node.parentNode;
-    pseudo = ":after";
+    pseudo = "::after";
   }
   return {
-    bindingElement: bindingElement,
-    pseudo: pseudo,
+    bindingElement,
+    pseudo,
   };
 }
 exports.getBindingElementAndPseudo = getBindingElementAndPseudo;
 
 /**
- * Returns css style rules for a given a node.
+ * Returns css rules for a given a node.
  * This function can handle ::before or ::after pseudo element as well as
  * normal element.
  */
-function getCSSStyleRules(node) {
+function getMatchingCSSRules(node) {
   const { bindingElement, pseudo } = getBindingElementAndPseudo(node);
-  const rules = InspectorUtils.getCSSStyleRules(bindingElement, pseudo);
+  const rules = InspectorUtils.getMatchingCSSRules(bindingElement, pseudo);
   return rules;
 }
-exports.getCSSStyleRules = getCSSStyleRules;
+exports.getMatchingCSSRules = getMatchingCSSRules;
+
+/**
+ * Returns true if the given node has visited state.
+ */
+function hasVisitedState(node) {
+  if (!node) {
+    return false;
+  }
+
+  // ElementState::VISITED
+  const ELEMENT_STATE_VISITED = 1 << 18;
+
+  return (
+    !!(InspectorUtils.getContentState(node) & ELEMENT_STATE_VISITED) ||
+    InspectorUtils.hasPseudoClassLock(node, ":visited")
+  );
+}
+exports.hasVisitedState = hasVisitedState;
+
+/**
+ * Find the position of [element] in [nodeList].
+ * @returns an index of the match, or -1 if there is no match
+ */
+function positionInNodeList(element, nodeList) {
+  for (let i = 0; i < nodeList.length; i++) {
+    if (element === nodeList[i]) {
+      return i;
+    }
+  }
+  return -1;
+}
+
+/**
+ * For a provided node, find the appropriate container/node couple so that
+ * container.contains(node) and a CSS selector can be created from the
+ * container to the node.
+ */
+function findNodeAndContainer(node) {
+  const shadowRoot = node.containingShadowRoot;
+  while (node?.isNativeAnonymous) {
+    node = node.parentNode;
+  }
+
+  if (shadowRoot) {
+    // If the node is under a shadow root, the shadowRoot contains the node and
+    // we can find the node via shadowRoot.querySelector(path).
+    return {
+      containingDocOrShadow: shadowRoot,
+      node,
+    };
+  }
+
+  // Otherwise, get the root binding parent to get a non anonymous element that
+  // will be accessible from the ownerDocument.
+  return {
+    containingDocOrShadow: node.ownerDocument,
+    node,
+  };
+}
+
+/**
+ * Find a unique CSS selector for a given element
+ * @returns a string such that:
+ *   - ele.containingDocOrShadow.querySelector(reply) === ele
+ *   - ele.containingDocOrShadow.querySelectorAll(reply).length === 1
+ */
+const findCssSelector = function (ele) {
+  const { node, containingDocOrShadow } = findNodeAndContainer(ele);
+  ele = node;
+
+  if (!containingDocOrShadow || !containingDocOrShadow.contains(ele)) {
+    // findCssSelector received element not inside container.
+    return "";
+  }
+
+  const cssEscape = ele.ownerGlobal.CSS.escape;
+
+  // document.querySelectorAll("#id") returns multiple if elements share an ID
+  if (
+    ele.id &&
+    containingDocOrShadow.querySelectorAll("#" + cssEscape(ele.id)).length === 1
+  ) {
+    return "#" + cssEscape(ele.id);
+  }
+
+  // Inherently unique by tag name
+  const tagName = ele.localName;
+  if (tagName === "html") {
+    return "html";
+  }
+  if (tagName === "head") {
+    return "head";
+  }
+  if (tagName === "body") {
+    return "body";
+  }
+
+  // We might be able to find a unique class name
+  let selector, index, matches;
+  for (let i = 0; i < ele.classList.length; i++) {
+    // Is this className unique by itself?
+    selector = "." + cssEscape(ele.classList.item(i));
+    matches = containingDocOrShadow.querySelectorAll(selector);
+    if (matches.length === 1) {
+      return selector;
+    }
+    // Maybe it's unique with a tag name?
+    selector = cssEscape(tagName) + selector;
+    matches = containingDocOrShadow.querySelectorAll(selector);
+    if (matches.length === 1) {
+      return selector;
+    }
+    // Maybe it's unique using a tag name and nth-child
+    index = positionInNodeList(ele, ele.parentNode.children) + 1;
+    selector = selector + ":nth-child(" + index + ")";
+    matches = containingDocOrShadow.querySelectorAll(selector);
+    if (matches.length === 1) {
+      return selector;
+    }
+  }
+
+  // Not unique enough yet.
+  index = positionInNodeList(ele, ele.parentNode.children) + 1;
+  selector = cssEscape(tagName) + ":nth-child(" + index + ")";
+  if (ele.parentNode !== containingDocOrShadow) {
+    selector = findCssSelector(ele.parentNode) + " > " + selector;
+  }
+  return selector;
+};
+exports.findCssSelector = findCssSelector;
+
+/**
+ * Get the full CSS path for a given element.
+ *
+ * @returns a string that can be used as a CSS selector for the element. It might not
+ * match the element uniquely. It does however, represent the full path from the root
+ * node to the element.
+ */
+function getCssPath(ele) {
+  const { node, containingDocOrShadow } = findNodeAndContainer(ele);
+  ele = node;
+  if (!containingDocOrShadow || !containingDocOrShadow.contains(ele)) {
+    // getCssPath received element not inside container.
+    return "";
+  }
+
+  const nodeGlobal = ele.ownerGlobal.Node;
+
+  const getElementSelector = element => {
+    if (!element.localName) {
+      return "";
+    }
+
+    let label =
+      element.nodeName == element.nodeName.toUpperCase()
+        ? element.localName.toLowerCase()
+        : element.localName;
+
+    if (element.id) {
+      label += "#" + element.id;
+    }
+
+    if (element.classList) {
+      for (const cl of element.classList) {
+        label += "." + cl;
+      }
+    }
+
+    return label;
+  };
+
+  const paths = [];
+
+  while (ele) {
+    if (!ele || ele.nodeType !== nodeGlobal.ELEMENT_NODE) {
+      break;
+    }
+
+    paths.splice(0, 0, getElementSelector(ele));
+    ele = ele.parentNode;
+  }
+
+  return paths.length ? paths.join(" ") : "";
+}
+exports.getCssPath = getCssPath;
+
+/**
+ * Get the xpath for a given element.
+ *
+ * @param {DomNode} ele
+ * @returns a string that can be used as an XPath to find the element uniquely.
+ */
+function getXPath(ele) {
+  const { node, containingDocOrShadow } = findNodeAndContainer(ele);
+  ele = node;
+  if (!containingDocOrShadow || !containingDocOrShadow.contains(ele)) {
+    // getXPath received element not inside container.
+    return "";
+  }
+
+  // Create a short XPath for elements with IDs.
+  if (ele.id) {
+    return `//*[@id="${ele.id}"]`;
+  }
+
+  // Otherwise walk the DOM up and create a part for each ancestor.
+  const parts = [];
+
+  const nodeGlobal = ele.ownerGlobal.Node;
+  // Use nodeName (instead of localName) so namespace prefix is included (if any).
+  while (ele && ele.nodeType === nodeGlobal.ELEMENT_NODE) {
+    let nbOfPreviousSiblings = 0;
+    let hasNextSiblings = false;
+
+    // Count how many previous same-name siblings the element has.
+    let sibling = ele.previousSibling;
+    while (sibling) {
+      // Ignore document type declaration.
+      if (
+        sibling.nodeType !== nodeGlobal.DOCUMENT_TYPE_NODE &&
+        sibling.nodeName == ele.nodeName
+      ) {
+        nbOfPreviousSiblings++;
+      }
+
+      sibling = sibling.previousSibling;
+    }
+
+    // Check if the element has at least 1 next same-name sibling.
+    sibling = ele.nextSibling;
+    while (sibling) {
+      if (sibling.nodeName == ele.nodeName) {
+        hasNextSiblings = true;
+        break;
+      }
+      sibling = sibling.nextSibling;
+    }
+
+    const prefix = ele.prefix ? ele.prefix + ":" : "";
+    const nth =
+      nbOfPreviousSiblings || hasNextSiblings
+        ? `[${nbOfPreviousSiblings + 1}]`
+        : "";
+
+    parts.push(prefix + ele.localName + nth);
+
+    ele = ele.parentNode;
+  }
+
+  return parts.length ? "/" + parts.reverse().join("/") : "";
+}
+exports.getXPath = getXPath;
+
+/**
+ * Build up a regular expression that matches a CSS variable token. This is an
+ * ident token that starts with two dashes "--".
+ *
+ * https://www.w3.org/TR/css-syntax-3/#ident-token-diagram
+ */
+var NON_ASCII = "[^\\x00-\\x7F]";
+var ESCAPE = "\\\\[^\n\r]";
+var VALID_CHAR = ["[_a-z0-9-]", NON_ASCII, ESCAPE].join("|");
+var IS_VARIABLE_TOKEN = new RegExp(`^--(${VALID_CHAR})*$`, "i");
+
+/**
+ * Check that this is a CSS variable.
+ *
+ * @param {String} input
+ * @return {Boolean}
+ */
+function isCssVariable(input) {
+  return !!input.match(IS_VARIABLE_TOKEN);
+}
+exports.isCssVariable = isCssVariable;
